#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Ansible module to manage CheckPoint Firewall (c) 2019
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
#

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {
    "metadata_version": "1.1",
    "status": ["preview"],
    "supported_by": "community",
}

DOCUMENTATION = """
---
module: cp_mgmt_add_domain
short_description: Create new object
description:
  - Create new object
  - All operations are performed over Web Services API.
version_added: "2.1.0"
author: "Or Soffer (@chkp-orso)"
options:
  name:
    description:
      - Object name.
    type: str
    required: True
  servers:
    description:
      - Domain servers. When this field is provided, 'set-domain' command is executed asynchronously.
    type: list
    elements: dict
    suboptions:
      name:
        description:
          - Object name. Must be unique in the domain.
        type: str
      ip_address:
        description:
          - IPv4 or IPv6 address. If both addresses are required use ipv4-address and ipv6-address fields explicitly.
        type: str
      ipv4_address:
        description:
          - IPv4 address.
        type: str
      ipv6_address:
        description:
          - IPv6 address.
        type: str
      multi_domain_server:
        description:
          - Multi Domain server name or UID.
        type: str
      active:
        description:
          - Activate domain server. Only one domain server is allowed to be active
        type: bool
      skip_start_domain_server:
        description:
          - Set this value to be true to prevent starting the new created domain.
        type: bool
      type:
        description:
          - Domain server type.
        type: str
        choices: ['management server', 'log server', 'smc']
  color:
    description:
      - Color of the object. Should be one of existing colors.
    type: str
    choices: ['aquamarine', 'black', 'blue', 'crete blue', 'burlywood', 'cyan', 'dark green', 'khaki', 'orchid', 'dark orange', 'dark sea green',
             'pink', 'turquoise', 'dark blue', 'firebrick', 'brown', 'forest green', 'gold', 'dark gold', 'gray', 'dark gray', 'light green', 'lemon chiffon',
             'coral', 'sea green', 'sky blue', 'magenta', 'purple', 'slate blue', 'violet red', 'navy blue', 'olive', 'orange', 'red', 'sienna', 'yellow']
  comments:
    description:
      - Comments string.
    type: str
  details_level:
    description:
      - The level of detail for some of the fields in the response can vary from showing only the UID value of the object to a fully detailed
        representation of the object.
    type: str
    choices: ['uid', 'standard', 'full']
  ignore_warnings:
    description:
      - Apply changes ignoring warnings.
    type: bool
  ignore_errors:
    description:
      - Apply changes ignoring errors. You won't be able to publish such a changes. If ignore-warnings flag was omitted - warnings will also be ignored.
    type: bool
extends_documentation_fragment: check_point.mgmt.checkpoint_commands
"""

EXAMPLES = """
- name: add-domain
  cp_mgmt_add_domain:
    name: domain1
    servers:
      ip_address: 192.0.2.1
      multi_domain_server: MDM_Server
      name: domain1_ManagementServer_1
"""

RETURN = """
cp_mgmt_domain:
  description: The checkpoint add-domain output.
  returned: always.
  type: dict
"""

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.check_point.mgmt.plugins.module_utils.checkpoint import (
    checkpoint_argument_spec_for_commands,
    api_command,
)


def main():
    argument_spec = dict(
        name=dict(type="str", required=True),
        servers=dict(
            type="list",
            elements="dict",
            options=dict(
                name=dict(type="str"),
                ip_address=dict(type="str"),
                ipv4_address=dict(type="str"),
                ipv6_address=dict(type="str"),
                multi_domain_server=dict(type="str"),
                active=dict(type="bool"),
                skip_start_domain_server=dict(type="bool"),
                type=dict(
                    type="str",
                    choices=["management server", "log server", "smc"],
                ),
            ),
        ),
        color=dict(
            type="str",
            choices=[
                "aquamarine",
                "black",
                "blue",
                "crete blue",
                "burlywood",
                "cyan",
                "dark green",
                "khaki",
                "orchid",
                "dark orange",
                "dark sea green",
                "pink",
                "turquoise",
                "dark blue",
                "firebrick",
                "brown",
                "forest green",
                "gold",
                "dark gold",
                "gray",
                "dark gray",
                "light green",
                "lemon chiffon",
                "coral",
                "sea green",
                "sky blue",
                "magenta",
                "purple",
                "slate blue",
                "violet red",
                "navy blue",
                "olive",
                "orange",
                "red",
                "sienna",
                "yellow",
            ],
        ),
        comments=dict(type="str"),
        details_level=dict(type="str", choices=["uid", "standard", "full"]),
        ignore_warnings=dict(type="bool"),
        ignore_errors=dict(type="bool"),
    )
    argument_spec.update(checkpoint_argument_spec_for_commands)

    module = AnsibleModule(argument_spec=argument_spec)
    command = "add-domain"

    result = api_command(module, command)
    module.exit_json(**result)


if __name__ == "__main__":
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import (absolute_import, division, print_function)
__metaclass__ = type
from ansible.plugins.action import ActionBase
try:
    from ansible_collections.ansible.utils.plugins.module_utils.common.argspec_validate import (
        AnsibleArgSpecValidator,
    )
except ImportError:
    ANSIBLE_UTILS_IS_INSTALLED = False
else:
    ANSIBLE_UTILS_IS_INSTALLED = True
from ansible.errors import AnsibleActionFail
from ansible_collections.cisco.dnac.plugins.plugin_utils.dnac import (
    DNACSDK,
    dnac_argument_spec,
    dnac_compare_equality,
    get_dict_result,
)
from ansible_collections.cisco.dnac.plugins.plugin_utils.exceptions import (
    AnsibleSDAException,
)

# Get common arguments specification
argument_spec = dnac_argument_spec()
# Add arguments specific for this module
argument_spec.update(dict(
    state=dict(type="str", default="present", choices=["present", "absent"]),
    payload=dict(type="list"),
    fabricId=dict(type="str"),
    networkDeviceId=dict(type="str"),
    id=dict(type="str"),
))

required_if = [
    ("state", "present", ["id"], True),
    ("state", "present", ["payload"], True),
    ("state", "absent", ["id"], True),
    ("state", "absent", ["payload"], True),
]
required_one_of = []
mutually_exclusive = []
required_together = []


class SdaFabricDevicesLayer2HandoffsV1(object):
    def __init__(self, params, dnac):
        self.dnac = dnac
        self.new_object = dict(
            payload=params.get("payload"),
            fabric_id=params.get("fabricId"),
            network_device_id=params.get("networkDeviceId"),
            id=params.get("id"),
        )

    def get_all_params(self, name=None, id=None):
        new_object_params = {}
        new_object_params['fabric_id'] = self.new_object.get('fabricId') or \
            self.new_object.get('fabric_id')
        new_object_params['network_device_id'] = self.new_object.get('networkDeviceId') or \
            self.new_object.get('network_device_id')
        new_object_params['offset'] = self.new_object.get('offset')
        new_object_params['limit'] = self.new_object.get('limit')
        return new_object_params

    def create_params(self):
        new_object_params = {}
        new_object_params['payload'] = self.new_object.get('payload')
        return new_object_params

    def delete_by_id_params(self):
        new_object_params = {}
        new_object_params['id'] = self.new_object.get('id')
        return new_object_params

    def get_object_by_name(self, name, is_absent=False):
        result = None
        # NOTE: Does not have a get by name method or it is in another action
        try:
            items = self.dnac.exec(
                family="sda",
                function="get_fabric_devices_layer2_handoffs_v1",
                params=self.get_all_params(name=name),
            )
            if isinstance(items, dict):
                if 'response' in items:
                    items = items.get('response')
                if isinstance(items, dict) and items.get("status") == "failed":
                    if is_absent:
                        raise AnsibleSDAException(response=items)
                    result = None
                    return result
            result = get_dict_result(items, 'name', name)
        except Exception:
            if is_absent:
                raise
            result = None
        return result

    def get_object_by_id(self, id):
        result = None
        # NOTE: Does not have a get by id method or it is in another action
        try:
            items = self.dnac.exec(
                family="sda",
                function="get_fabric_devices_layer2_handoffs_v1",
                params=self.get_all_params(id=id),
            )
            if isinstance(items, dict):
                if 'response' in items:
                    items = items.get('response')
            result = get_dict_result(items, 'id', id)
        except Exception:
            result = None
        return result

    def exists(self, is_absent=False):
        name = self.new_object.get("name")
        prev_obj = self.get_object_by_name(name, is_absent=is_absent)
        it_exists = prev_obj is not None and isinstance(prev_obj, dict) and prev_obj.get("status") != "failed"
        return (it_exists, prev_obj)

    def requires_update(self, current_obj):
        requested_obj = self.new_object.get('payload')
        if requested_obj and len(requested_obj) > 0:
            requested_obj = requested_obj[0]

        obj_params = [
            ("networkDeviceId", "networkDeviceId"),
            ("fabricId", "fabricId"),
            ("interfaceName", "interfaceName"),
            ("internalVlanId", "internalVlanId"),
            ("externalVlanId", "externalVlanId"),
            ("fabricId", "fabric_id"),
            ("networkDeviceId", "network_device_id"),
            ("id", "id"),
        ]
        # Method 1. Params present in request (Ansible) obj are the same as the current (DNAC) params
        # If any does not have eq params, it requires update
        return any(not dnac_compare_equality(current_obj.get(dnac_param),
                                             requested_obj.get(ansible_param))
                   for (dnac_param, ansible_param) in obj_params)

    def create(self):
        result = self.dnac.exec(
            family="sda",
            function="add_fabric_devices_layer2_handoffs_v1",
            params=self.create_params(),
            op_modifies=True,
        )
        if isinstance(result, dict):
            if 'response' in result:
                result = result.get('response')
            if isinstance(result, dict) and result.get("status") == "failed":
                raise AnsibleSDAException(response=result)
        return result

    def update(self):
        requested_obj = self.new_object.get('payload')
        if requested_obj and len(requested_obj) > 0:
            requested_obj = requested_obj[0]
        id = self.new_object.get("id") or requested_obj.get("id")
        name = self.new_object.get("name") or requested_obj.get("name")
        result = None
        # NOTE: Does not have update method. What do we do?
        return result

    def delete(self):
        requested_obj = self.new_object.get('payload')
        if requested_obj and len(requested_obj) > 0:
            requested_obj = requested_obj[0]
        id = self.new_object.get("id") or requested_obj.get("id")
        name = self.new_object.get("name") or requested_obj.get("name")
        result = None
        if not id:
            prev_obj_name = self.get_object_by_name(name)
            id_ = None
            if prev_obj_name:
                id_ = prev_obj_name.get("id")
            if id_:
                self.new_object.update(dict(id=id_))
        result = self.dnac.exec(
            family="sda",
            function="delete_fabric_device_layer2_handoff_by_id_v1",
            params=self.delete_by_id_params(),
        )
        return result


class ActionModule(ActionBase):
    def __init__(self, *args, **kwargs):
        if not ANSIBLE_UTILS_IS_INSTALLED:
            raise AnsibleActionFail("ansible.utils is not installed. Execute 'ansible-galaxy collection install ansible.utils'")
        super(ActionModule, self).__init__(*args, **kwargs)
        self._supports_async = False
        self._supports_check_mode = False
        self._result = None

    # Checks the supplied parameters against the argument spec for this module
    def _check_argspec(self):
        aav = AnsibleArgSpecValidator(
            data=self._task.args,
            schema=dict(argument_spec=argument_spec),
            schema_format="argspec",
            schema_conditionals=dict(
                required_if=required_if,
                required_one_of=required_one_of,
                mutually_exclusive=mutually_exclusive,
                required_together=required_together,
            ),
            name=self._task.action,
        )
        valid, errors, self._task.args = aav.validate()
        if not valid:
            raise AnsibleActionFail(errors)

    def run(self, tmp=None, task_vars=None):
        self._task.diff = False
        self._result = super(ActionModule, self).run(tmp, task_vars)
        self._result["changed"] = False
        self._check_argspec()

        dnac = DNACSDK(self._task.args)
        obj = SdaFabricDevicesLayer2HandoffsV1(self._task.args, dnac)

        state = self._task.args.get("state")

        response = None

        if state == "present":
            (obj_exists, prev_obj) = obj.exists()
            if obj_exists:
                if obj.requires_update(prev_obj):
                    response = obj.update()
                    dnac.object_updated()
                else:
                    response = prev_obj
                    dnac.object_already_present()
            else:
                try:
                    response = obj.create()
                    dnac.object_created()
                except AnsibleSDAException as e:
                    dnac.fail_json("Could not create object {e}".format(e=e._response))

        elif state == "absent":
            try:
                (obj_exists, prev_obj) = obj.exists(is_absent=True)
                if obj_exists:
                    response = obj.delete()
                    dnac.object_deleted()
                else:
                    dnac.object_already_absent()
            except AnsibleSDAException as e:
                dnac.fail_json("Could not get object to be delete {e}".format(e=e._response))

        self._result.update(dict(dnac_response=response))
        self._result.update(dnac.exit_json())
        return self._result



#!/usr/bin/python
#
# Copyright (c) 2020 Paul Aiton, (@paultaiton)
# Copyright (c) 2018 Yunge Zhu, (@yungezz)
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
---
module: azure_rm_roleassignment
version_added: "0.1.2"
short_description: Manage Azure Role Assignment
description:
    - Create and delete instance of Azure Role Assignment.

options:
    assignee_object_id:
        description:
            - The object id of assignee. This maps to the ID inside the Active Directory.
            - It can point to a user, service principal or security group.
            - Required when creating role assignment.
        type: str
        aliases:
          - assignee
    id:
        description:
            - Fully qualified id of assignment to delete or create.
            - Mutually Exclusive with I(scope) and I(name)
        type: str
    name:
        description:
            - Unique name of role assignment.
            - The role assignment name must be a GUID, sample as "3ce0cbb0-58c4-4e6d-a16d-99d86a78b3ca".
            - Mutually Exclusive with I(id)
        type: str
    role_definition_id:
        description:
            - The role definition id used in the role assignment.
            - Required when creating role assignment.
        type: str
    scope:
        description:
            - The scope of the role assignment to create.
            - For example, use /subscriptions/{subscription-id}/ for subscription.
            - /subscriptions/{subscription-id}/resourceGroups/{resource-group-name} for resource group.
            - /subscriptions/{subscription-id}/resourceGroups/{resource-group-name}/providers/{resource-provider}/{resource-type}/{resource-name} for resource.
            - Mutually Exclusive with I(id)
        type: str
    state:
        description:
            - Assert the state of the role assignment.
            - Use C(present) to create or update a role assignment and C(absent) to delete it.
            - If C(present), then I(role_definition_id) and I(assignee_object_id) are both required
        default: present
        type: str
        choices:
            - absent
            - present

extends_documentation_fragment:
    - azure.azcollection.azure

author:
    - Yunge Zhu(@yungezz)
    - Paul Aiton(@paultaiton)

'''

EXAMPLES = '''
- name: Create a role assignment
  azure_rm_roleassignment:
    scope: /subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx
    assignee_object_id: xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx
    role_definition_id:
      "/subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/providers/Microsoft.Authorization/roleDefinitions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx"

- name: Create a role assignment
  azure_rm_roleassignment:
    name: xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx
    scope: /subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx
    assignee_object_id: xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx
    role_definition_id:
      "/subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/providers/Microsoft.Authorization/roleDefinitions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx"

- name: Delete a role assignment
  azure_rm_roleassignment:
    name: xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx
    scope: /subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx
    state: absent

- name: Delete a role assignment
  azure_rm_roleassignment:
    id: /subscriptions/xxx-sub-guid-xxx/resourceGroups/rgname/providers/Microsoft.Authorization/roleAssignments/xxx-assign-guid-xxx"
    state: absent

- name: Delete a role assignment
  azure_rm_roleassignment:
    scope: /subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx
    assignee_object_id: xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx
    role_definition_id:
      "/subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/providers/Microsoft.Authorization/roleDefinitions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx"
    state: absent
'''

RETURN = '''
id:
    description:
        - Id of current role assignment.
    returned: always
    type: str
    sample: "/subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/providers/Microsoft.Authorization/roleAssignments/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx"
name:
    description:
        - Name of role assignment.
    type: str
    returned: always
    sample: xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx
type:
    description:
        - Type of role assignment.
    type: str
    returned: always
    sample: Microsoft.Authorization/roleAssignments
assignee_object_id:
    description:
        - Principal Id of the role assignee.
    type: str
    returned: always
    sample: xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx
principal_type:
    description:
        - Principal type of the role assigned to.
    type: str
    returned: always
    sample: ServicePrincipal
role_definition_id:
    description:
        - Role definition id that was assigned to principal_id.
    type: str
    returned: always
    sample: /subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/providers/Microsoft.Authorization/roleDefinitions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx
scope:
    description:
        - The role assignment scope.
    type: str
    returned: always
    sample: /subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx
'''

try:
    import uuid
    from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common import AzureRMModuleBase

except ImportError:
    # This is handled in azure_rm_common
    pass


class AzureRMRoleAssignment(AzureRMModuleBase):
    """Configuration class for an Azure RM Role Assignment"""

    def __init__(self):
        self.module_arg_spec = dict(
            assignee_object_id=dict(type='str', aliases=['assignee']),
            id=dict(type='str'),
            name=dict(type='str'),
            role_definition_id=dict(type='str'),
            scope=dict(type='str'),
            state=dict(type='str', default='present', choices=['present', 'absent'])
        )

        self.assignee_object_id = None
        self.id = None
        self.name = None
        self.role_definition_id = None
        self.scope = None
        self.state = None

        self.results = dict(
            changed=False,
            id=None,
        )

        mutually_exclusive = [['name', 'id'], ['scope', 'id']]
        required_one_of = [['scope', 'id']]
        required_if = [
            ["state", "present", ["assignee_object_id", "role_definition_id"]]
        ]

        super(AzureRMRoleAssignment, self).__init__(derived_arg_spec=self.module_arg_spec,
                                                    supports_check_mode=True,
                                                    supports_tags=False,
                                                    required_one_of=required_one_of,
                                                    required_if=required_if,
                                                    mutually_exclusive=mutually_exclusive)

    def exec_module(self, **kwargs):
        """Main module execution method"""

        for key in self.module_arg_spec:
            setattr(self, key, kwargs[key])

        if self.name and not self.scope:
            self.fail("Parameter Error: setting name requires a scope to also be set.")

        existing_assignment = None
        response = None

        existing_assignment = self.get_roleassignment()

        if existing_assignment:
            self.set_results(existing_assignment)

        if self.state == 'present':
            # check if the role assignment exists
            if not existing_assignment:
                self.log("Role assignment doesn't exist in this scope")

                self.results['changed'] = True

                if self.check_mode:
                    return self.results
                response = self.create_roleassignment()
                self.set_results(response)

            else:
                self.log("Role assignment already exists, not updatable")
                self.log('Result: {0}'.format(existing_assignment))

        elif self.state == 'absent':
            if existing_assignment:
                self.log("Delete role assignment")
                self.results['changed'] = True

                if self.check_mode:
                    return self.results

                self.delete_roleassignment(existing_assignment.get('id'))

                self.log('role assignment deleted')

            else:
                # If assignment doesn't exist, that's the desired state.
                self.log("role assignment {0} does not exist.".format(self.name))

        return self.results

    def create_roleassignment(self):
        '''
        Creates role assignment.

        :return: deserialized role assignment
        '''
        self.log("Creating role assignment {0}".format(self.name))

        response = None
        try:
            # pylint: disable=missing-kwoa
            parameters = self.authorization_models.RoleAssignmentCreateParameters(role_definition_id=self.role_definition_id,
                                                                                  principal_id=self.assignee_object_id)
            if self.id:
                response = self.authorization_client.role_assignments.create_by_id(role_id=self.id,
                                                                                   parameters=parameters)
            elif self.scope:
                if not self.name:
                    self.name = str(uuid.uuid4())
                response = self.authorization_client.role_assignments.create(scope=self.scope,
                                                                             role_assignment_name=self.name,
                                                                             parameters=parameters)

        except Exception as exc:
            self.log('Error attempting to create role assignment.')
            self.fail("Error creating role assignment: {0}".format(str(exc)))
        return self.roleassignment_to_dict(response)

    def delete_roleassignment(self, assignment_id):
        '''
        Deletes specified role assignment.

        :return: True
        '''
        self.log("Deleting the role assignment {0}".format(self.name))
        try:
            response = self.authorization_client.role_assignments.delete_by_id(role_id=assignment_id)
        except Exception as e:
            self.log('Error attempting to delete the role assignment.')
            self.fail("Error deleting the role assignment: {0}".format(str(e)))

        return True

    def get_roleassignment(self):
        '''
        Gets the properties of the specified role assignment.

        :return: deserialized role assignment dictionary
        '''
        self.log("Checking if the role assignment {0} is present".format(self.name))

        role_assignment = None

        if self.id:
            try:
                response = self.authorization_client.role_assignments.get_by_id(role_id=self.id)
                role_assignment = self.roleassignment_to_dict(response)
                if role_assignment and self.assignee_object_id and role_assignment.get('assignee_object_id') != self.assignee_object_id:
                    self.fail('State Mismatch Error: The assignment ID exists, but does not match the provided assignee.')

                if role_assignment and self.role_definition_id and (role_assignment.get('role_definition_id').split('/')[-1].lower()
                                                                    != self.role_definition_id.split('/')[-1].lower()):
                    self.fail('State Mismatch Error: The assignment ID exists, but does not match the provided role.')

            except Exception as ex:
                self.log("Didn't find role assignments id {0}".format(self.id))

        elif self.name and self.scope:
            try:
                response = self.authorization_client.role_assignments.get(scope=self.scope, role_assignment_name=self.name)
                role_assignment = self.roleassignment_to_dict(response)
                if role_assignment and self.assignee_object_id and role_assignment.get('assignee_object_id') != self.assignee_object_id:
                    self.fail('State Mismatch Error: The assignment name exists, but does not match the provided assignee.')

                if role_assignment and self.role_definition_id and (role_assignment.get('role_definition_id').split('/')[-1].lower()
                                                                    != self.role_definition_id.split('/')[-1].lower()):
                    self.fail('State Mismatch Error: The assignment name exists, but does not match the provided role.')

            except Exception as ex:
                self.log("Didn't find role assignment by name {0} at scope {1}".format(self.name, self.scope))

        else:
            try:
                if self.scope and self.assignee_object_id and self.role_definition_id:
                    response = list(self.authorization_client.role_assignments.list())
                    response = [self.roleassignment_to_dict(role_assignment) for role_assignment in response]
                    response = [role_assignment for role_assignment in response if role_assignment.get('scope') == self.scope]
                    response = [role_assignment for role_assignment in response if role_assignment.get('assignee_object_id') == self.assignee_object_id]
                    response = [role_assignment for role_assignment in response if (role_assignment.get('role_definition_id').split('/')[-1].lower()
                                                                                    == self.role_definition_id.split('/')[-1].lower())]
                else:
                    self.fail('If id or name are not supplied, then assignee_object_id and role_definition_id are required.')
                if response:
                    role_assignment = response[0]
            except Exception as ex:
                self.log("Didn't find role assignments for subscription {0}".format(self.subscription_id))

        return role_assignment

    def set_results(self, assignment):
        self.results['id'] = assignment.get('id')
        self.results['name'] = assignment.get('name')
        self.results['type'] = assignment.get('type')
        self.results['assignee_object_id'] = assignment.get('assignee_object_id')
        self.results['principal_type'] = assignment.get('principal_type')
        self.results['role_definition_id'] = assignment.get('role_definition_id')
        self.results['scope'] = assignment.get('scope')

    def roleassignment_to_dict(self, assignment):
        return dict(
            assignee_object_id=assignment.principal_id,
            id=assignment.id,
            name=assignment.name,
            principal_id=assignment.principal_id,
            principal_type=assignment.principal_type,
            role_definition_id=assignment.role_definition_id,
            scope=assignment.scope,
            type=assignment.type
        )


def main():
    """Main execution"""
    AzureRMRoleAssignment()


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)


DOCUMENTATION = r"""
---
module: lan_automation_update
short_description: Resource module for Lan Automation Update
description:
- This module represents an alias of the module lan_automation_update_v1
version_added: '6.14.0'
extends_documentation_fragment:
  - cisco.dnac.module
author: Rafael Campos (@racampos)
options:
  id:
    description: Id path parameter. LAN Automation id can be obtained from /dna/intent/api/v1/lan-automation/status.
    type: str
  payload:
    description: Lan Automation Update's payload.
    elements: dict
    suboptions:
      deviceManagementIPAddress:
        description: Device Management IP Address.
        type: str
      newLoopback0IPAddress:
        description: New Loopback0 IP Address from LAN pool of Device Discovery Site.
        type: str
    type: list
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for LAN Automation LANAutomationStopAndUpdateDevicesV1
  description: Complete reference of the LANAutomationStopAndUpdateDevicesV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!l-an-automation-stop-and-update-devices
notes:
  - SDK Method used are
    lan_automation.LanAutomation.lan_automation_stop_and_update_devices_v1,

  - Paths used are
    put /dna/intent/api/v1/lan-automation/{id},
  - It should be noted that this module is an alias of lan_automation_update_v1

"""

EXAMPLES = r"""
- name: Update by id
  cisco.dnac.lan_automation_update:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    id: string
    payload:
    - deviceManagementIPAddress: string
      newLoopback0IPAddress: string

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": {
        "taskId": "string",
        "url": "string"
      },
      "version": "string"
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: node_deployment
short_description: Resource module for Node Deployment
description:
- Manage operations create, update and delete of the resource Node Deployment.
- This API registers a Cisco ISE node to form a multi-node deployment.
- The deregistered node becomes a standalone Cisco ISE node.
- This API updates the configuration of the Cisco ISE node with the.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.ise.module
author: Rafael Campos (@racampos)
options:
  allowCertImport:
    description: Consent to import the self-signed certificate of the registering node.
    type: bool
  fqdn:
    description: Node Deployment's fqdn.
    type: str
  hostname:
    description: Hostname path parameter. Hostname of the deployed node.
    type: str
  password:
    description: Node Deployment's password.
    type: str
  roles:
    description: Roles can be empty or have many values for a node.
    elements: str
    type: list
  services:
    description: Services can be empty or have many values for a node.
    elements: str
    type: list
  userName:
    description: Node Deployment's userName.
    type: str
requirements:
- ciscoisesdk >= 2.2.3
- python >= 3.5
seealso:
- name: Cisco ISE documentation for Node Deployment
  description: Complete reference of the Node Deployment API.
  link: https://developer.cisco.com/docs/identity-services-engine/v1/#!deployment-openapi
notes:
  - SDK Method used are
    node_deployment.NodeDeployment.delete_node,
    node_deployment.NodeDeployment.register_node,
    node_deployment.NodeDeployment.update_node,

  - Paths used are
    post /api/v1/deployment/node,
    delete /api/v1/deployment/node/{hostname},
    put /api/v1/deployment/node/{hostname},

"""

EXAMPLES = r"""
- name: Create
  cisco.ise.node_deployment:
    ise_hostname: "{{ise_hostname}}"
    ise_username: "{{ise_username}}"
    ise_password: "{{ise_password}}"
    ise_verify: "{{ise_verify}}"
    state: present
    allowCertImport: true
    fqdn: string
    password: string
    roles:
    - string
    services:
    - string
    userName: string

- name: Update by name
  cisco.ise.node_deployment:
    ise_hostname: "{{ise_hostname}}"
    ise_username: "{{ise_username}}"
    ise_password: "{{ise_password}}"
    ise_verify: "{{ise_verify}}"
    state: present
    hostname: string
    roles:
    - string
    services:
    - string

- name: Delete by name
  cisco.ise.node_deployment:
    ise_hostname: "{{ise_hostname}}"
    ise_username: "{{ise_username}}"
    ise_password: "{{ise_password}}"
    ise_verify: "{{ise_verify}}"
    state: absent
    hostname: string

"""

RETURN = r"""
ise_response:
  description: A dictionary or list with the response returned by the Cisco ISE Python SDK
  returned: always
  type: dict
  sample: >
    {
      "fqdn": "string",
      "hostname": "string",
      "ipAddress": "string",
      "nodeStatus": "string",
      "roles": [
        "string"
      ],
      "services": [
        "string"
      ]
    }

ise_update_response:
  description: A dictionary or list with the response returned by the Cisco ISE Python SDK
  returned: always
  version_added: '1.1.0'
  type: dict
  sample: >
    {
      "success": {
        "message": "string"
      },
      "version": "string"
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)


DOCUMENTATION = r"""
---
module: planned_access_points
short_description: Resource module for Planned Access Points
description:
- This module represents an alias of the module planned_access_points_v1
version_added: '6.0.0'
extends_documentation_fragment:
  - cisco.dnac.module
author: Rafael Campos (@racampos)
options:
  attributes:
    description: Planned Access Points's attributes.
    suboptions:
      createDate:
        description: Created date of the planned access point.
        type: float
      domain:
        description: Service domain to which the planned access point belongs.
        type: str
      heirarchyName:
        description: Hierarchy name of the planned access point.
        type: str
      id:
        description: Unique id of the planned access point.
        type: float
      instanceUuid:
        description: Instance uuid of the planned access point.
        type: str
      macAddress:
        description: MAC address of the planned access point.
        type: str
      name:
        description: Display name of the planned access point.
        type: str
      source:
        description: Source of the data used to create the planned access point.
        type: str
      typeString:
        description: Type string representation of the planned access point.
        type: str
    type: dict
  floorId:
    description: FloorId path parameter. The instance UUID of the floor hierarchy element.
    type: str
  isSensor:
    description: Indicates that PAP is a sensor.
    type: bool
  location:
    description: Planned Access Points's location.
    suboptions:
      altitude:
        description: Altitude of the planned access point's location.
        type: float
      lattitude:
        description: Latitude of the planned access point's location.
        type: float
      longtitude:
        description: Longitude of the planned access point's location.
        type: float
    type: dict
  plannedAccessPointUuid:
    description: PlannedAccessPointUuid path parameter. The instance UUID of the planned
      access point to delete.
    type: str
  position:
    description: Planned Access Points's position.
    suboptions:
      x:
        description: X-coordinate of the planned access point on the map, 0,0 point
          being the top-left corner.
        type: float
      y:
        description: Y-coordinate of the planned access point on the map, 0,0 point
          being the top-left corner.
        type: float
      z:
        description: Z-coordinate, or height, of the planned access point on the map.
        type: float
    type: dict
  radioCount:
    description: Number of radios of the planned access point.
    type: int
  radios:
    description: Planned Access Points's radios.
    elements: dict
    suboptions:
      antenna:
        description: Planned Access Points's antenna.
        suboptions:
          azimuthAngle:
            description: Azimuth angle of the antenna.
            type: float
          elevationAngle:
            description: Elevation angle of the antenna.
            type: float
          gain:
            description: Gain of the antenna.
            type: float
          mode:
            description: Mode of the antenna associated with this radio.
            type: str
          name:
            description: Name of the antenna.
            type: str
          type:
            description: Type of the antenna associated with this radio.
            type: str
        type: dict
      attributes:
        description: Planned Access Points's attributes.
        suboptions:
          channel:
            description: Channel in which this radio operates.
            type: float
          channelString:
            description: Channel string representation.
            type: str
          id:
            description: Id of the radio.
            type: int
          ifMode:
            description: IF mode of the radio.
            type: str
          ifTypeString:
            description: String representation of native band.
            type: str
          ifTypeSubband:
            description: Sub band of the radio.
            type: str
          instanceUuid:
            description: Instance Uuid of the radio.
            type: str
          slotId:
            description: Slot number in which the radio resides in the parent access
              point.
            type: float
          txPowerLevel:
            description: Tx Power at which this radio operates (in dBm).
            type: float
        type: dict
      isSensor:
        description: Determines if it is sensor or not.
        type: bool
    type: list
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Devices CreatePlannedAccessPointForFloorV1
  description: Complete reference of the CreatePlannedAccessPointForFloorV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!create-planned-access-point-for-floor
- name: Cisco DNA Center documentation for Devices DeletePlannedAccessPointForFloorV1
  description: Complete reference of the DeletePlannedAccessPointForFloorV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!delete-planned-access-point-for-floor
- name: Cisco DNA Center documentation for Devices UpdatePlannedAccessPointForFloorV1
  description: Complete reference of the UpdatePlannedAccessPointForFloorV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!update-planned-access-point-for-floor
notes:
  - SDK Method used are
    devices.Devices.create_planned_access_point_for_floor_v1,
    devices.Devices.delete_planned_access_point_for_floor_v1,
    devices.Devices.update_planned_access_point_for_floor_v1,

  - Paths used are
    post /dna/intent/api/v1/floors/{floorId}/planned-access-points,
    delete /dna/intent/api/v1/floors/{floorId}/planned-access-points/{plannedAccessPointUuid},
    put /dna/intent/api/v1/floors/{floorId}/planned-access-points,
  - It should be noted that this module is an alias of planned_access_points_v1

"""

EXAMPLES = r"""
- name: Update all
  cisco.dnac.planned_access_points:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    state: present
    attributes:
      createDate: 0
      domain: string
      heirarchyName: string
      id: 0
      instanceUuid: string
      macAddress: string
      name: string
      source: string
      typeString: string
    floorId: string
    isSensor: true
    location:
      altitude: 0
      lattitude: 0
      longtitude: 0
    position:
      x: 0
      y: 0
      z: 0
    radioCount: 0
    radios:
    - antenna:
        azimuthAngle: 0
        elevationAngle: 0
        gain: 0
        mode: string
        name: string
        type: string
      attributes:
        channel: 0
        channelString: string
        id: 0
        ifMode: string
        ifTypeString: string
        ifTypeSubband: string
        instanceUuid: string
        slotId: 0
        txPowerLevel: 0
      isSensor: true

- name: Create
  cisco.dnac.planned_access_points:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    state: present
    attributes:
      createDate: 0
      domain: string
      heirarchyName: string
      id: 0
      instanceUuid: string
      macAddress: string
      name: string
      source: string
      typeString: string
    floorId: string
    isSensor: true
    location:
      altitude: 0
      lattitude: 0
      longtitude: 0
    position:
      x: 0
      y: 0
      z: 0
    radioCount: 0
    radios:
    - antenna:
        azimuthAngle: 0
        elevationAngle: 0
        gain: 0
        mode: string
        name: string
        type: string
      attributes:
        channel: 0
        channelString: string
        id: 0
        ifMode: string
        ifTypeString: string
        ifTypeSubband: string
        instanceUuid: string
        slotId: 0
        txPowerLevel: 0
      isSensor: true

- name: Delete by id
  cisco.dnac.planned_access_points:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    state: absent
    floorId: string
    plannedAccessPointUuid: string

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "version": "string",
      "response": {
        "url": "string",
        "taskId": "string"
      }
    }
"""



#!/usr/bin/python
# coding: utf-8 -*-


# (c) 2020, John Westcott IV <john.westcott.iv@redhat.com>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type


ANSIBLE_METADATA = {'metadata_version': '1.1', 'status': ['preview'], 'supported_by': 'community'}

DOCUMENTATION = '''
---
module: workflow_job_template_node
author: "John Westcott IV (@john-westcott-iv)"
short_description: create, update, or destroy Automation Platform Controller workflow job template nodes.
description:
    - Create, update, or destroy Automation Platform Controller workflow job template nodes.
    - Use this to build a graph for a workflow, which dictates what the workflow runs.
    - You can create nodes first, and link them afterwards, and not worry about ordering.
      For failsafe referencing of a node, specify identifier, WFJT, and organization.
      With those specified, you can choose to modify or not modify any other parameter.
options:
    extra_data:
      description:
        - Variables to apply at launch time.
        - Will only be accepted if job template prompts for vars or has a survey asking for those vars.
      type: dict
    inventory:
      description:
        - Name, ID, or named URL of the Inventory applied as a prompt, if job template prompts for inventory
      type: str
    scm_branch:
      description:
        - SCM branch applied as a prompt, if job template prompts for SCM branch
      type: str
    job_type:
      description:
        - Job type applied as a prompt, if job template prompts for job type
      type: str
      choices:
        - 'run'
        - 'check'
    job_tags:
      description:
        - Job tags applied as a prompt, if job template prompts for job tags
      type: str
    skip_tags:
      description:
        - Tags to skip, applied as a prompt, if job tempalte prompts for job tags
      type: str
    limit:
      description:
        - Limit to act on, applied as a prompt, if job template prompts for limit
      type: str
    diff_mode:
      description:
        - Run diff mode, applied as a prompt, if job template prompts for diff mode
      type: bool
    verbosity:
      description:
        - Verbosity applied as a prompt, if job template prompts for verbosity
      type: str
      choices:
        - '0'
        - '1'
        - '2'
        - '3'
        - '4'
        - '5'
    workflow_job_template:
      description:
        - The workflow job template name, ID, or named URL the node exists in.
        - Used for looking up the node, cannot be modified after creation.
      required: True
      type: str
      aliases:
        - workflow
    organization:
      description:
        - The organization name, ID, or named URL of the workflow job template the node exists in.
        - Used for looking up the workflow, not a direct model field.
      type: str
    unified_job_template:
      description:
        - Name of unified job template to run in the workflow.
        - Can be a job template, project, inventory source, etc.
        - Omit if creating an approval node.
        - This parameter is mutually exclusive with C(approval_node).
      type: str
    lookup_organization:
      description:
        - Organization name, ID, or named URL the inventories, job template, project, inventory source the unified_job_template exists in.
        - If not provided, will lookup by name only, which does not work with duplicates.
      type: str
    approval_node:
      description:
        - A dictionary of Name, description, and timeout values for the approval node.
        - This parameter is mutually exclusive with C(unified_job_template).
      type: dict
      suboptions:
        name:
          description:
            - Name of this workflow approval template.
          type: str
          required: True
        description:
          description:
            - Optional description of this workflow approval template.
          type: str
        timeout:
          description:
            - The amount of time (in seconds) before the approval node expires and fails.
          type: int
    all_parents_must_converge:
      description:
        - If enabled then the node will only run if all of the parent nodes have met the criteria to reach this node
      type: bool
    identifier:
      description:
        - An identifier for this node that is unique within its workflow.
        - It is copied to workflow job nodes corresponding to this node.
      required: True
      type: str
    always_nodes:
      description:
        - Nodes that will run after this node completes.
        - List of node identifiers.
      type: list
      elements: str
    success_nodes:
      description:
        - Nodes that will run after this node on success.
        - List of node identifiers.
      type: list
      elements: str
    failure_nodes:
      description:
        - Nodes that will run after this node on failure.
        - List of node identifiers.
      type: list
      elements: str
    credentials:
      description:
        - Credential names, IDs, or named URLs to be applied to job as launch-time prompts.
        - List of credential names, IDs, or named URLs.
        - Uniqueness is not handled rigorously.
      type: list
      elements: str
    execution_environment:
      description:
        - Execution Environment name, ID, or named URL applied as a prompt, assuming job template prompts for execution environment
      type: str
    forks:
      description:
        - Forks applied as a prompt, assuming job template prompts for forks
      type: int
    instance_groups:
      description:
        - List of Instance Group names, IDs, or named URLs applied as a prompt, assuming job template prompts for instance groups
      type: list
      elements: str
    job_slice_count:
      description:
        - Job Slice Count applied as a prompt, assuming job template prompts for job slice count
      type: int
    labels:
      description:
        - List of labels applied as a prompt, assuming job template prompts for labels
      type: list
      elements: str
    timeout:
      description:
        - Timeout applied as a prompt, assuming job template prompts for timeout
      type: int
    state:
      description:
        - Desired state of the resource.
      choices: ["present", "absent", "exists"]
      default: "present"
      type: str
extends_documentation_fragment: awx.awx.auth
'''

EXAMPLES = '''
- name: Create a node, follows workflow_job_template example
  workflow_job_template_node:
    identifier: my-first-node
    workflow: example-workflow
    unified_job_template: jt-for-node-use
    organization: Default  # organization of workflow job template
    extra_data:
      foo_key: bar_value

- name: Create parent node for prior node
  workflow_job_template_node:
    identifier: my-root-node
    workflow: example-workflow
    unified_job_template: jt-for-node-use
    organization: Default
    success_nodes:
      - my-first-node

- name: Create workflow with 2 Job Templates and an approval node in between
  block:
  - name: Create a workflow job template
    tower_workflow_job_template:
      name: my-workflow-job-template
      ask_scm_branch_on_launch: true
      organization: Default

  - name: Create 1st node
    tower_workflow_job_template_node:
      identifier: my-first-node
      workflow_job_template: my-workflow-job-template
      unified_job_template: some_job_template
      organization: Default

  - name: Create 2nd approval node
    tower_workflow_job_template_node:
      identifier: my-second-approval-node
      workflow_job_template: my-workflow-job-template
      organization: Default
      approval_node:
        description: "Do this?"
        name: my-second-approval-node
        timeout: 3600

  - name: Create 3rd node
    tower_workflow_job_template_node:
      identifier: my-third-node
      workflow_job_template: my-workflow-job-template
      unified_job_template: some_other_job_template
      organization: Default

  - name: Link 1st node to 2nd Approval node
    tower_workflow_job_template_node:
      identifier: my-first-node
      workflow_job_template: my-workflow-job-template
      organization: Default
      success_nodes:
        - my-second-approval-node

  - name: Link 2nd Approval Node 3rd node
    tower_workflow_job_template_node:
      identifier: my-second-approval-node
      workflow_job_template: my-workflow-job-template
      organization: Default
      success_nodes:
        - my-third-node
'''

from ..module_utils.controller_api import ControllerAPIModule


def main():
    # Any additional arguments that are not fields of the item can be added here
    argument_spec = dict(
        identifier=dict(required=True),
        workflow_job_template=dict(required=True, aliases=['workflow']),
        organization=dict(),
        extra_data=dict(type='dict'),
        inventory=dict(),
        scm_branch=dict(),
        job_type=dict(choices=['run', 'check']),
        job_tags=dict(),
        skip_tags=dict(),
        limit=dict(),
        diff_mode=dict(type='bool'),
        verbosity=dict(choices=['0', '1', '2', '3', '4', '5']),
        unified_job_template=dict(),
        lookup_organization=dict(),
        approval_node=dict(type='dict'),
        all_parents_must_converge=dict(type='bool'),
        success_nodes=dict(type='list', elements='str'),
        always_nodes=dict(type='list', elements='str'),
        failure_nodes=dict(type='list', elements='str'),
        credentials=dict(type='list', elements='str'),
        execution_environment=dict(type='str'),
        forks=dict(type='int'),
        instance_groups=dict(type='list', elements='str'),
        job_slice_count=dict(type='int'),
        labels=dict(type='list', elements='str'),
        timeout=dict(type='int'),
        state=dict(choices=['present', 'absent', 'exists'], default='present'),
    )
    mutually_exclusive = [("unified_job_template", "approval_node")]
    required_if = [
        ['state', 'absent', ['identifier']],
        ['state', 'present', ['identifier']],
        ['state', 'present', ['unified_job_template', 'approval_node', 'success_nodes', 'always_nodes', 'failure_nodes'], True],
    ]

    # Create a module for ourselves
    module = ControllerAPIModule(
        argument_spec=argument_spec,
        mutually_exclusive=mutually_exclusive,
        required_if=required_if,
    )

    # Extract our parameters
    identifier = module.params.get('identifier')
    state = module.params.get('state')
    approval_node = module.params.get('approval_node')
    new_fields = {}
    lookup_organization = module.params.get('lookup_organization')
    search_fields = {'identifier': identifier}

    # Attempt to look up the related items the user specified (these will fail the module if not found)
    workflow_job_template = module.params.get('workflow_job_template')
    workflow_job_template_id = None
    if workflow_job_template:
        wfjt_search_fields = {}
        organization = module.params.get('organization')
        if organization:
            organization_id = module.resolve_name_to_id('organizations', organization)
            wfjt_search_fields['organization'] = organization_id
        wfjt_data = module.get_one('workflow_job_templates', name_or_id=workflow_job_template, **{'data': wfjt_search_fields})
        if wfjt_data is None:
            module.fail_json(
                msg="The workflow {0} in organization {1} was not found on the controller instance server".format(workflow_job_template, organization)
            )
        workflow_job_template_id = wfjt_data['id']
        search_fields['workflow_job_template'] = new_fields['workflow_job_template'] = workflow_job_template_id

    # Attempt to look up an existing item based on the provided data
    existing_item = module.get_one('workflow_job_template_nodes', check_exists=(state == 'exists'), **{'data': search_fields})

    if state == 'absent':
        # If the state was absent we can let the module delete it if needed, the module will handle exiting from this
        module.delete_if_needed(existing_item)

    # Set lookup data to use
    search_fields = {}
    if lookup_organization:
        search_fields['organization'] = module.resolve_name_to_id('organizations', lookup_organization)

    unified_job_template = module.params.get('unified_job_template')
    if unified_job_template:
        new_fields['unified_job_template'] = module.get_one('unified_job_templates', name_or_id=unified_job_template, **{'data': search_fields})['id']
    inventory = module.params.get('inventory')
    if inventory:
        new_fields['inventory'] = module.resolve_name_to_id('inventories', inventory)

    # Create the data that gets sent for create and update
    for field_name in (
        'identifier',
        'extra_data',
        'scm_branch',
        'job_type',
        'job_tags',
        'skip_tags',
        'limit',
        'diff_mode',
        'verbosity',
        'all_parents_must_converge',
        'forks',
        'job_slice_count',
        'timeout',
    ):
        field_val = module.params.get(field_name)
        if field_val:
            new_fields[field_name] = field_val

    association_fields = {}
    for association in ('always_nodes', 'success_nodes', 'failure_nodes', 'credentials', 'instance_groups', 'labels'):
        name_list = module.params.get(association)
        if name_list is None:
            continue
        id_list = []
        for sub_name in name_list:
            if association in ['credentials', 'instance_groups', 'labels']:
                sub_obj = module.get_one(association, name_or_id=sub_name)
            else:
                endpoint = 'workflow_job_template_nodes'
                lookup_data = {'identifier': sub_name}
                if workflow_job_template_id:
                    lookup_data['workflow_job_template'] = workflow_job_template_id
                sub_obj = module.get_one(endpoint, **{'data': lookup_data})
            if sub_obj is None:
                module.fail_json(msg='Could not find {0} entry with name {1}'.format(association, sub_name))
            id_list.append(sub_obj['id'])
        association_fields[association] = id_list

    execution_environment = module.params.get('execution_environment')
    if execution_environment is not None:
        if execution_environment == '':
            new_fields['execution_environment'] = ''
        else:
            ee = module.get_one('execution_environments', name_or_id=execution_environment)
            if ee is None:
                module.fail_json(msg='could not find execution_environment entry with name {0}'.format(execution_environment))
            else:
                new_fields['execution_environment'] = ee['id']

    # In the case of a new object, the utils need to know it is a node
    new_fields['type'] = 'workflow_job_template_node'

    # If the state was present and we can let the module build or update the existing item, this will return on its own
    module.create_or_update_if_needed(
        existing_item,
        new_fields,
        endpoint='workflow_job_template_nodes',
        item_type='workflow_job_template_node',
        auto_exit=not approval_node,
        associations=association_fields,
    )

    # Create approval node unified template or update existing
    if approval_node:
        # Set Approval Fields
        new_fields = {}

        # Extract Parameters
        if approval_node.get('name') is None:
            module.fail_json(msg="Approval node name is required to create approval node.")
        if approval_node.get('name') is not None:
            new_fields['name'] = approval_node['name']
        if approval_node.get('description') is not None:
            new_fields['description'] = approval_node['description']
        if approval_node.get('timeout') is not None:
            new_fields['timeout'] = approval_node['timeout']

        # Find created workflow node ID
        search_fields = {'identifier': identifier}
        search_fields['workflow_job_template'] = workflow_job_template_id
        workflow_job_template_node = module.get_one('workflow_job_template_nodes', **{'data': search_fields})
        workflow_job_template_node_id = workflow_job_template_node['id']
        module.json_output['workflow_node_id'] = workflow_job_template_node_id
        existing_item = None
        # Due to not able to lookup workflow_approval_templates, find the existing item in another place
        if workflow_job_template_node['related'].get('unified_job_template') is not None:
            existing_item = module.get_endpoint(workflow_job_template_node['related']['unified_job_template'])['json']
        approval_endpoint = 'workflow_job_template_nodes/{0}/create_approval_template/'.format(workflow_job_template_node_id)
        module.create_or_update_if_needed(
            existing_item, new_fields, endpoint=approval_endpoint, item_type='workflow_job_template_approval_node', associations=association_fields
        )
    module.exit_json(**module.json_output)


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: sda_anycast_gateways_v1
short_description: Resource module for Sda Anycast Gateways V1
description:
- Manage operations create, update and delete of the resource Sda Anycast Gateways V1.
- Adds anycast gateways based on user input.
- Deletes an anycast gateway based on id.
- Updates anycast gateways based on user input.
version_added: '6.14.0'
extends_documentation_fragment:
  - cisco.dnac.module
author: Rafael Campos (@racampos)
options:
  id:
    description: Id path parameter. ID of the anycast gateway.
    type: str
  payload:
    description: Sda Anycast Gateways's payload.
    elements: dict
    suboptions:
      fabricId:
        description: ID of the fabric this anycast gateway is assigned to. Updating
          anycast gateways on fabric zones is not allowed--instead, update the corresponding
          anycast gateway on the fabric site and the updates will be applied on all
          applicable fabric zones (updating this field is not allowed).
        type: str
      id:
        description: ID of the anycast gateway (updating this field is not allowed).
        type: str
      ipPoolName:
        description: Name of the IP pool associated with the anycast gateway (updating
          this field is not allowed).
        type: str
      isCriticalPool:
        description: Enable/disable critical VLAN (not applicable to INFRA_VN; updating
          this field is not allowed).
        type: bool
      isGroupBasedPolicyEnforcementEnabled:
        description: Enable/disable Group-Based Policy Enforcement (applicable only
          to INFRA_VN; defaults to false).
        type: bool
      isIntraSubnetRoutingEnabled:
        description: Enable/disable Intra-Subnet Routing (not applicable to INFRA_VN;
          updating this field is not allowed).
        type: bool
      isIpDirectedBroadcast:
        description: Enable/disable IP-directed broadcast (not applicable to INFRA_VN).
        type: bool
      isLayer2FloodingEnabled:
        description: Enable/disable layer 2 flooding (not applicable to INFRA_VN).
        type: bool
      isMultipleIpToMacAddresses:
        description: Enable/disable multiple IP-to-MAC Addresses (Wireless Bridged-Network
          Virtual Machine; not applicable to INFRA_VN).
        type: bool
      isSupplicantBasedExtendedNodeOnboarding:
        description: Enable/disable Supplicant-Based Extended Node Onboarding (applicable
          only to INFRA_VN requests; must not be null when poolType is EXTENDED_NODE).
        type: bool
      isWirelessPool:
        description: Enable/disable fabric-enabled wireless (not applicable to INFRA_VN).
        type: bool
      poolType:
        description: The pool type of the anycast gateway (required for & applicable
          only to INFRA_VN; updating this field is not allowed).
        type: str
      securityGroupName:
        description: Name of the associated Security Group (not applicable to INFRA_VN).
        type: str
      tcpMssAdjustment:
        description: TCP maximum segment size adjustment.
        type: int
      trafficType:
        description: The type of traffic the anycast gateway serves.
        type: str
      virtualNetworkName:
        description: Name of the layer 3 virtual network associated with the anycast
          gateway (updating this field is not allowed).
        type: str
      vlanId:
        description: ID of the VLAN of the anycast gateway (updating this field is not
          allowed).
        type: int
      vlanName:
        description: Name of the VLAN of the anycast gateway (updating this field is
          not allowed).
        type: str
    type: list
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for SDA AddAnycastGatewaysV1
  description: Complete reference of the AddAnycastGatewaysV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!add-anycast-gateways
- name: Cisco DNA Center documentation for SDA DeleteAnycastGatewayByIdV1
  description: Complete reference of the DeleteAnycastGatewayByIdV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!delete-anycast-gateway-by-id
- name: Cisco DNA Center documentation for SDA UpdateAnycastGatewaysV1
  description: Complete reference of the UpdateAnycastGatewaysV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!update-anycast-gateways
notes:
  - SDK Method used are
    sda.Sda.add_anycast_gateways_v1,
    sda.Sda.delete_anycast_gateway_by_id_v1,
    sda.Sda.update_anycast_gateways_v1,

  - Paths used are
    post /dna/intent/api/v1/sda/anycastGateways,
    delete /dna/intent/api/v1/sda/anycastGateways/{id},
    put /dna/intent/api/v1/sda/anycastGateways,

"""

EXAMPLES = r"""
- name: Update all
  cisco.dnac.sda_anycast_gateways_v1:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    state: present
    payload:
    - fabricId: string
      id: string
      ipPoolName: string
      isCriticalPool: true
      isGroupBasedPolicyEnforcementEnabled: true
      isIntraSubnetRoutingEnabled: true
      isIpDirectedBroadcast: true
      isLayer2FloodingEnabled: true
      isMultipleIpToMacAddresses: true
      isSupplicantBasedExtendedNodeOnboarding: true
      isWirelessPool: true
      poolType: string
      securityGroupName: string
      tcpMssAdjustment: 0
      trafficType: string
      virtualNetworkName: string
      vlanId: 0
      vlanName: string

- name: Create
  cisco.dnac.sda_anycast_gateways_v1:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    state: present
    payload:
    - autoGenerateVlanName: true
      fabricId: string
      ipPoolName: string
      isCriticalPool: true
      isGroupBasedPolicyEnforcementEnabled: true
      isIntraSubnetRoutingEnabled: true
      isIpDirectedBroadcast: true
      isLayer2FloodingEnabled: true
      isMultipleIpToMacAddresses: true
      isSupplicantBasedExtendedNodeOnboarding: true
      isWirelessPool: true
      poolType: string
      securityGroupName: string
      tcpMssAdjustment: 0
      trafficType: string
      virtualNetworkName: string
      vlanId: 0
      vlanName: string

- name: Delete by id
  cisco.dnac.sda_anycast_gateways_v1:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    state: absent
    id: string

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": {
        "taskId": "string",
        "url": "string"
      },
      "version": "string"
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: site_create_v1
short_description: Resource module for Site Create V1
description:
- Manage operation create of the resource Site Create V1.
- Creates site with area/building/floor with specified hierarchy.
version_added: '3.1.0'
extends_documentation_fragment:
  - cisco.dnac.module
author: Rafael Campos (@racampos)
options:
  headers:
    description: Additional headers.
    type: dict
  site:
    description: Site Create's site.
    suboptions:
      area:
        description: Site Create's area.
        suboptions:
          name:
            description: Name of the area (eg Area1).
            type: str
          parentName:
            description: Parent name of the area to be created.
            type: str
        type: dict
      building:
        description: Site Create's building.
        suboptions:
          address:
            description: Address of the building to be created.
            type: str
          country:
            description: Country (eg United States).
            type: str
          latitude:
            description: Latitude coordinate of the building (eg 37.338).
            type: float
          longitude:
            description: Longitude coordinate of the building (eg -121.832).
            type: float
          name:
            description: Name of the building (eg building1).
            type: str
          parentName:
            description: Parent name of building to be created.
            type: str
        type: dict
      floor:
        description: Site Create's floor.
        suboptions:
          floorNumber:
            description: Floor number. (eg 5).
            type: float
          height:
            description: Height of the floor. Unit of measure is ft. (eg 15).
            type: float
          length:
            description: Length of the floor. Unit of measure is ft. (eg 100).
            type: float
          name:
            description: Name of the floor (eg floor-1).
            type: str
          parentName:
            description: Parent name of the floor to be created.
            type: str
          rfModel:
            description: Type of floor (eg Cubes And Walled Offices0.
            type: str
          width:
            description: Width of the floor. Unit of measure is ft. (eg 100).
            type: float
        type: dict
    type: dict
  type:
    description: Type of site to create (eg area, building, floor).
    type: str
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Sites CreateSiteV1
  description: Complete reference of the CreateSiteV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!create-site
notes:
  - SDK Method used are
    sites.Sites.create_site_v1,

  - Paths used are
    post /dna/intent/api/v1/site,

"""

EXAMPLES = r"""
- name: Create
  cisco.dnac.site_create_v1:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: '{{my_headers | from_json}}'
    site:
      area:
        name: string
        parentName: string
      building:
        address: string
        country: string
        latitude: 0
        longitude: 0
        name: string
        parentName: string
      floor:
        floorNumber: 0
        height: 0
        length: 0
        name: string
        parentName: string
        rfModel: string
        width: 0
    type: string

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "executionId": "string",
      "executionStatusUrl": "string",
      "message": "string"
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: aci_bindings_info
short_description: Information module for ACI Bindings
description:
- Get all ACI Bindings.
- This API allows clients to retrieve all the bindings that were sent to Cisco.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.ise.module_info
author: Rafael Campos (@racampos)
options:
  page:
    description:
    - Page query parameter. Page number.
    type: int
  size:
    description:
    - Size query parameter. Number of objects returned per page.
    type: int
  sort:
    description:
    - Sort query parameter. Sort type - asc or desc.
    type: str
  sortBy:
    description:
    - SortBy query parameter. Sort column by which objects needs to be sorted.
    type: str
  filterBy:
    description:
    - FilterBy query parameter.
    elements: str
    type: list
  filterValue:
    description:
    - FilterValue query parameter.
    elements: str
    type: list
requirements:
- ciscoisesdk >= 2.2.3
- python >= 3.5
notes:
  - SDK Method used are
    aci_bindings.AciBindings.get_aci_bindings_generator,

  - Paths used are
    get /ers/config/acibindings/getall,

"""

EXAMPLES = r"""
- name: Get all ACI Bindings
  cisco.ise.aci_bindings_info:
    ise_hostname: "{{ise_hostname}}"
    ise_username: "{{ise_username}}"
    ise_password: "{{ise_password}}"
    ise_verify: "{{ise_verify}}"
    page: 1
    size: 20
    sort: asc
    sortBy: string
    filterBy: []
    filterValue: []
  register: result

"""

RETURN = r"""
ise_response:
  description: A dictionary or list with the response returned by the Cisco ISE Python SDK
  returned: always
  type: dict
  sample: >
    {
      "id": "string",
      "name": "string",
      "description": "string",
      "ip": "string",
      "sgtValue": "string",
      "vn": "string",
      "psn": "string",
      "learnedFrom": "string",
      "learnedBy": "string"
    }

ise_responses:
  description: A dictionary or list with the response returned by the Cisco ISE Python SDK
  returned: always
  version_added: '1.1.0'
  type: list
  elements: dict
  sample: >
    [
      {
        "id": "string",
        "name": "string",
        "description": "string",
        "ip": "string",
        "sgtValue": "string",
        "vn": "string",
        "psn": "string",
        "learnedFrom": "string",
        "learnedBy": "string"
      }
    ]
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: duo_identitysync_sync_info
short_description: Information module for Duo Identitysync Sync Info
description:
- Get Duo Identitysync Sync Info by name.
- Initiate the sync between the Active Directory and the corresponding Mfa provider associated with this Identitysync config.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.ise.module_info
author: Rafael Campos (@racampos)
options:
  syncName:
    description:
    - SyncName path parameter. Name of the Identitysync configuration used to initiate sync.
    type: str
requirements:
- ciscoisesdk >= 2.2.3
- python >= 3.5
notes:
  - SDK Method used are
    duo_identity_sync.DuoIdentitySync.sync,

  - Paths used are
    get /api/v1/duo-identitysync/identitysync/sync/{syncName},

"""

EXAMPLES = r"""
- name: Get Duo Identitysync Sync Info by name
  cisco.ise.duo_identitysync_sync_info:
    ise_hostname: "{{ise_hostname}}"
    ise_username: "{{ise_username}}"
    ise_password: "{{ise_password}}"
    ise_verify: "{{ise_verify}}"
    syncName: string
  register: result

"""

RETURN = r"""
ise_response:
  description: A dictionary or list with the response returned by the Cisco ISE Python SDK
  returned: always
  type: str
  sample: >
    "'string'"
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: network_profiles_for_sites_site_assignments_bulk_create_v1
short_description: Resource module for Network Profiles For Sites Site Assignments Bulk Create V1
description:
- Manage operation create of the resource Network Profiles For Sites Site Assignments Bulk Create V1.
- Assign a network profile for sites to a list of sites. Also assigns the profile to child sites.
version_added: '6.15.0'
extends_documentation_fragment:
  - cisco.dnac.module
author: Rafael Campos (@racampos)
options:
  profileId:
    description: ProfileId path parameter. The `id` of the network profile, retrievable
      from `GET /intent/api/v1/networkProfilesForSites`.
    type: str
  type:
    description: Network Profiles For Sites Site Assignments Bulk Create's type.
    type: dict
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Site Design AssignANetworkProfileForSitesToAListOfSitesV1
  description: Complete reference of the AssignANetworkProfileForSitesToAListOfSitesV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!assign-a-network-profile-for-sites-to-a-list-of-sites
notes:
  - SDK Method used are
    site_design.SiteDesign.assign_a_network_profile_for_sites_to_a_list_of_sites_v1,

  - Paths used are
    post /dna/intent/api/v1/networkProfilesForSites/{profileId}/siteAssignments/bulk,

"""

EXAMPLES = r"""
- name: Create
  cisco.dnac.network_profiles_for_sites_site_assignments_bulk_create_v1:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    profileId: string
    type: {}

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "version": "string",
      "response": {
        "url": "string",
        "taskId": "string"
      }
    }
"""



#!/usr/bin/python
#
# Copyright (c) 2022 Andrea Decorte, <adecorte@redhat.com>
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
---
module: azure_rm_natgateway_info
version_added: "1.13.0"
short_description: Retrieve NAT Gateway instance facts
description:
    - Get facts for a NAT Gateway instance.
options:
    name:
        description:
            - Only show results for a specific NAT gateway.
        type: str
    resource_group:
        description:
            - Limit results by resource group.
        type: str

extends_documentation_fragment:
    - azure.azcollection.azure

author:
    - Andrea Decorte (@andreadecorte)
'''

EXAMPLES = '''
- name: Get facts for NAT gateway by name.
  azure_rm_natgateway_info:
    name: Mynatgw
    resource_group: MyResourceGroup

- name: Get facts for all NAT gateways in resource group.
  azure_rm_natgateway_info:
    resource_group: MyResourceGroup

- name: Get facts for all NAT gateways.
  azure_rm_natgateway_info:
'''

RETURN = '''
gateways:
    description:
        - A list of dictionaries containing facts for a NAT gateway.
    returned: always
    type: list
    elements: dict
    contains:
        id:
            description:
                - NAT gateway resource ID.
            returned: always
            type: str
            sample: /subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/myResourceGroup/providers/Microsoft.Network/natGateways/mynatgw
        name:
            description:
                - Name of NAT gateway.
            returned: always
            type: str
            sample: mynatgw
        resource_group:
            description:
                - Name of resource group.
            returned: always
            type: str
            sample: myResourceGroup
        location:
            description:
                - Location of NAT gateway.
            returned: always
            type: str
            sample: centralus
        idle_timeout_in_minutes:
            description:
                - The idle timeout of the NAT gateway.
            returned: always
            type: int
            sample: 4
        sku:
            description:
                - SKU of the NAT gateway.
            returned: always
            type: dict
            contains:
                name:
                    description:
                        - The name of the SKU.
                    returned: always
                    type: str
                    sample: Standard
        zones:
            description:
                - Availability Zones of the NAT gateway.
            returned: always
            type: list
            elements: str
        public_ip_addresses:
            description:
                - List of ids of public IP addresses associated to the NAT Gateway.
            returned: always
            type: list
            elements: str
'''

from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common import AzureRMModuleBase

try:
    from azure.core.exceptions import ResourceNotFoundError
    from azure.mgmt.core.tools import parse_resource_id
except ImportError:
    # This is handled in azure_rm_common
    pass


class AzureRMNATGatewayInfo(AzureRMModuleBase):

    def __init__(self):
        self.module_arg_spec = dict(
            name=dict(type='str'),
            resource_group=dict(type='str'),
        )

        self.results = dict(
            changed=False,
        )

        self.name = None
        self.resource_group = None

        super(AzureRMNATGatewayInfo, self).__init__(self.module_arg_spec,
                                                    supports_check_mode=True,
                                                    supports_tags=False,
                                                    facts_module=True)

    def exec_module(self, **kwargs):
        for key in self.module_arg_spec:
            setattr(self, key, kwargs[key])

        if self.name is not None:
            self.results["gateways"] = self.get()
        elif self.resource_group is not None:
            self.results["gateways"] = self.list_by_rg()
        else:
            self.results["gateways"] = self.list_all()

        return self.results

    def get(self):
        response = None
        results = []
        try:
            response = self.network_client.nat_gateways.get(resource_group_name=self.resource_group, nat_gateway_name=self.name)
        except ResourceNotFoundError:
            pass

        if response is not None:
            results.append(self.format_response(response))

        return results

    def list_by_rg(self):
        response = None
        results = []
        try:
            response = self.network_client.nat_gateways.list(resource_group_name=self.resource_group)
        except Exception as exc:
            request_id = exc.request_id if exc.request_id else ''
            self.fail("Error listing NAT gateways in resource groups {0}: {1} - {2}".format(self.resource_group, request_id, str(exc)))

        for item in response:
            results.append(self.format_response(item))

        return results

    def list_all(self):
        response = None
        results = []
        try:
            response = self.network_client.nat_gateways.list_all()
        except Exception as exc:
            request_id = exc.request_id if exc.request_id else ''
            self.fail("Error listing all NAT gateways: {0} - {1}".format(request_id, str(exc)))

        for item in response:
            results.append(self.format_response(item))

        return results

    def format_response(self, natgw):
        d = natgw.as_dict()
        id = d.get("id")
        id_dict = parse_resource_id(id)
        d = {
            "id": id,
            "name": d.get("name"),
            "resource_group": id_dict.get("resource_group", self.resource_group),
            "location": d.get("location"),
            "sku": d.get("sku"),
            "zones": d.get("zones"),
            "idle_timeout_in_minutes": d.get("idle_timeout_in_minutes"),
            "public_ip_addresses": d.get("public_ip_addresses")
        }
        return d


def main():
    AzureRMNATGatewayInfo()


if __name__ == "__main__":
    main()



from __future__ import absolute_import, division, print_function

__metaclass__ = type

import pytest

from awx.main.models import Inventory


@pytest.mark.django_db
def test_inventory_create(run_module, admin_user, organization):
    # Create an insights credential

    result = run_module(
        'inventory',
        {
            'name': 'foo-inventory',
            'organization': organization.name,
            'variables': {'foo': 'bar', 'another-foo': {'barz': 'bar2'}},
            'state': 'present',
        },
        admin_user,
    )
    assert not result.get('failed', False), result.get('msg', result)

    inv = Inventory.objects.get(name='foo-inventory')
    assert inv.variables == '{"foo": "bar", "another-foo": {"barz": "bar2"}}'

    result.pop('module_args', None)
    result.pop('invocation', None)
    assert result == {"name": "foo-inventory", "id": inv.id, "changed": True}

    assert inv.organization_id == organization.id


@pytest.mark.django_db
def test_invalid_smart_inventory_create(run_module, admin_user, organization):
    result = run_module(
        'inventory',
        {'name': 'foo-inventory', 'organization': organization.name, 'kind': 'smart', 'host_filter': 'ansible', 'state': 'present'},
        admin_user,
    )
    assert result.get('failed', False), result

    assert 'Invalid query ansible' in result['msg']


@pytest.mark.django_db
def test_valid_smart_inventory_create(run_module, admin_user, organization):
    result = run_module(
        'inventory',
        {'name': 'foo-inventory', 'organization': organization.name, 'kind': 'smart', 'host_filter': 'name=my_host', 'state': 'present'},
        admin_user,
    )
    assert not result.get('failed', False), result

    inv = Inventory.objects.get(name='foo-inventory')
    assert inv.host_filter == 'name=my_host'
    assert inv.kind == 'smart'
    assert inv.organization_id == organization.id



#!/usr/bin/python
#
# Copyright (c) 2018 Yuwei Zhou, <yuwzho@microsoft.com>
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
---
module: azure_rm_route
version_added: "0.1.2"
short_description: Manage Azure route resource
description:
    - Create, update or delete a route.
options:
    resource_group:
        description:
            - Name of resource group.
        required: true
        type: str
    name:
        description:
            - Name of the route.
        required: true
        type: str
    state:
        description:
            - Assert the state of the route. Use C(present) to create or update and C(absent) to delete.
        default: present
        type: str
        choices:
            - absent
            - present
    address_prefix:
        description:
            - The destination CIDR to which the route applies.
        type: str
    next_hop_type:
        description:
            - The type of Azure hop the packet should be sent to.
        type: str
        choices:
            - virtual_network_gateway
            - vnet_local
            - internet
            - virtual_appliance
            - none
        default: 'none'
    next_hop_ip_address:
        description:
            - The IP address packets should be forwarded to.
            - Next hop values are only allowed in routes where the next hop type is VirtualAppliance.
        type: str
    route_table_name:
        description:
            - The name of the route table.
        required: true
        type: str


extends_documentation_fragment:
    - azure.azcollection.azure
    - azure.azcollection.azure_tags

author:
    - Yuwei Zhou (@yuwzho)

'''

EXAMPLES = '''
- name: Create a route
  azure_rm_route:
    resource_group: myResourceGroup
    name: myRoute
    address_prefix: 10.1.0.0/16
    next_hop_type: virtual_network_gateway
    route_table_name: table

- name: Delete a route
  azure_rm_route:
    resource_group: myResourceGroup
    name: myRoute
    route_table_name: table
    state: absent
'''
RETURN = '''
id:
    description:
        - Current state of the route.
    returned: success
    type: str
    sample: "/subscriptions/xxxx...xxxx/resourceGroups/v-xisuRG/providers/Microsoft.Network/routeTables/tableb57/routes/routeb57"
'''

try:
    from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common import AzureRMModuleBase
    from azure.core.exceptions import ResourceNotFoundError
except ImportError:
    # This is handled in azure_rm_common
    pass

from ansible.module_utils.common.dict_transformations import _snake_to_camel


class AzureRMRoute(AzureRMModuleBase):

    def __init__(self):

        self.module_arg_spec = dict(
            resource_group=dict(type='str', required=True),
            name=dict(type='str', required=True),
            state=dict(type='str', default='present', choices=['present', 'absent']),
            address_prefix=dict(type='str'),
            next_hop_type=dict(type='str',
                               choices=['virtual_network_gateway',
                                        'vnet_local',
                                        'internet',
                                        'virtual_appliance',
                                        'none'],
                               default='none'),
            next_hop_ip_address=dict(type='str'),
            route_table_name=dict(type='str', required=True)
        )

        required_if = [
            ('state', 'present', ['next_hop_type'])
        ]

        self.resource_group = None
        self.name = None
        self.state = None
        self.address_prefix = None
        self.next_hop_type = None
        self.next_hop_ip_address = None
        self.route_table_name = None

        self.results = dict(
            changed=False,
            id=None
        )

        super(AzureRMRoute, self).__init__(self.module_arg_spec,
                                           required_if=required_if,
                                           supports_check_mode=True)

    def exec_module(self, **kwargs):

        for key in list(self.module_arg_spec.keys()):
            setattr(self, key, kwargs[key])

        result = dict()
        changed = False

        self.next_hop_type = _snake_to_camel(self.next_hop_type, capitalize_first=True)

        result = self.get_route()
        if self.state == 'absent' and result:
            changed = True
            if not self.check_mode:
                self.delete_route()
        elif self.state == 'present':
            if not result:
                changed = True  # create new route
            else:  # check update
                if result.next_hop_type != self.next_hop_type:
                    self.log('Update: {0} next_hop_type from {1} to {2}'.format(self.name, result.next_hop_type, self.next_hop_type))
                    changed = True
                if result.next_hop_ip_address != self.next_hop_ip_address:
                    self.log('Update: {0} next_hop_ip_address from {1} to {2}'.format(self.name, result.next_hop_ip_address, self.next_hop_ip_address))
                    changed = True
                if result.address_prefix != self.address_prefix:
                    self.log('Update: {0} address_prefix from {1} to {2}'.format(self.name, result.address_prefix, self.address_prefix))
                    changed = True
            if changed:
                result = self.network_models.Route(name=self.name,
                                                   address_prefix=self.address_prefix,
                                                   next_hop_type=self.next_hop_type,
                                                   next_hop_ip_address=self.next_hop_ip_address)
                if not self.check_mode:
                    result = self.create_or_update_route(result)

        self.results['id'] = result.id if result else None
        self.results['changed'] = changed
        return self.results

    def create_or_update_route(self, param):
        try:
            poller = self.network_client.routes.begin_create_or_update(self.resource_group, self.route_table_name, self.name, param)
            return self.get_poller_result(poller)
        except Exception as exc:
            self.fail("Error creating or updating route {0} - {1}".format(self.name, str(exc)))

    def delete_route(self):
        try:
            poller = self.network_client.routes.begin_delete(self.resource_group, self.route_table_name, self.name)
            result = self.get_poller_result(poller)
            return result
        except Exception as exc:
            self.fail("Error deleting route {0} - {1}".format(self.name, str(exc)))

    def get_route(self):
        try:
            return self.network_client.routes.get(self.resource_group, self.route_table_name, self.name)
        except ResourceNotFoundError as cloud_err:
            # Return None iff the resource is not found
            if cloud_err.status_code == 404:
                self.log('{0}'.format(str(cloud_err)))
                return None
            self.fail('Error: failed to get resource {0} - {1}'.format(self.name, str(cloud_err)))
        except Exception as exc:
            self.fail('Error: failed to get resource {0} - {1}'.format(self.name, str(exc)))


def main():
    AzureRMRoute()


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: active_directory_info
short_description: Information module for Active Directory
description:
- Get all Active Directory.
- Get Active Directory by id.
- Get Active Directory by name.
- This API allows the client to get Active Directory by name.
- This API fetchs the join point details by ID. The ID can be retrieved with the.
- This API lists all the join points for Active Directory domains in Cisco ISE.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.ise.module_info
author: Rafael Campos (@racampos)
options:
  name:
    description:
    - Name path parameter.
    type: str
  id:
    description:
    - Id path parameter.
    type: str
  page:
    description:
    - Page query parameter. Page number.
    type: int
  size:
    description:
    - Size query parameter. Number of objects returned per page.
    type: int
requirements:
- ciscoisesdk >= 2.2.3
- python >= 3.5
notes:
  - SDK Method used are
    active_directory.ActiveDirectory.get_active_directory_by_id,
    active_directory.ActiveDirectory.get_active_directory_by_name,
    active_directory.ActiveDirectory.get_active_directory_generator,

  - Paths used are
    get /ers/config/activedirectory,
    get /ers/config/activedirectory/name/{name},
    get /ers/config/activedirectory/{id},

"""

EXAMPLES = r"""
- name: Get all Active Directory
  cisco.ise.active_directory_info:
    ise_hostname: "{{ise_hostname}}"
    ise_username: "{{ise_username}}"
    ise_password: "{{ise_password}}"
    ise_verify: "{{ise_verify}}"
    page: 1
    size: 20
  register: result

- name: Get Active Directory by id
  cisco.ise.active_directory_info:
    ise_hostname: "{{ise_hostname}}"
    ise_username: "{{ise_username}}"
    ise_password: "{{ise_password}}"
    ise_verify: "{{ise_verify}}"
    id: string
  register: result

- name: Get Active Directory by name
  cisco.ise.active_directory_info:
    ise_hostname: "{{ise_hostname}}"
    ise_username: "{{ise_username}}"
    ise_password: "{{ise_password}}"
    ise_verify: "{{ise_verify}}"
    name: string
  register: result

"""

RETURN = r"""
ise_response:
  description: A dictionary or list with the response returned by the Cisco ISE Python SDK
  returned: always
  type: dict
  sample: >
    {
      "id": "string",
      "name": "string",
      "description": "string",
      "domain": "string",
      "enableDomainWhiteList": true,
      "enableDomainAllowedList": true,
      "adgroups": {
        "groups": [
          {
            "name": "string",
            "sid": "string",
            "type": "string"
          }
        ]
      },
      "advancedSettings": {
        "enablePassChange": true,
        "enableMachineAuth": true,
        "enableMachineAccess": true,
        "agingTime": 0,
        "enableDialinPermissionCheck": true,
        "enableCallbackForDialinClient": true,
        "plaintextAuth": true,
        "enableFailedAuthProtection": true,
        "authProtectionType": "string",
        "failedAuthThreshold": 0,
        "identityNotInAdBehaviour": "string",
        "unreachableDomainsBehaviour": "string",
        "enableRewrites": true,
        "rewriteRules": [
          {
            "rowId": 0,
            "rewriteMatch": "string",
            "rewriteResult": "string"
          }
        ],
        "firstName": "string",
        "department": "string",
        "lastName": "string",
        "organizationalUnit": "string",
        "jobTitle": "string",
        "locality": "string",
        "email": "string",
        "stateOrProvince": "string",
        "telephone": "string",
        "country": "string",
        "streetAddress": "string",
        "schema": "string"
      },
      "adAttributes": {
        "attributes": [
          {
            "name": "string",
            "type": "string",
            "internalName": "string",
            "defaultValue": "string"
          }
        ]
      },
      "adScopesNames": "string",
      "link": {
        "rel": "string",
        "href": "string",
        "type": "string"
      }
    }

ise_responses:
  description: A dictionary or list with the response returned by the Cisco ISE Python SDK
  returned: always
  version_added: '1.1.0'
  type: list
  elements: dict
  sample: >
    [
      {
        "id": "string",
        "name": "string",
        "description": "string",
        "domain": "string",
        "enableDomainWhiteList": true,
        "enableDomainAllowedList": true,
        "adgroups": {
          "groups": [
            {
              "name": "string",
              "sid": "string",
              "type": "string"
            }
          ]
        },
        "advancedSettings": {
          "enablePassChange": true,
          "enableMachineAuth": true,
          "enableMachineAccess": true,
          "agingTime": 0,
          "enableDialinPermissionCheck": true,
          "enableCallbackForDialinClient": true,
          "plaintextAuth": true,
          "enableFailedAuthProtection": true,
          "authProtectionType": "string",
          "failedAuthThreshold": 0,
          "identityNotInAdBehaviour": "string",
          "unreachableDomainsBehaviour": "string",
          "enableRewrites": true,
          "rewriteRules": [
            {
              "rowId": 0,
              "rewriteMatch": "string",
              "rewriteResult": "string"
            }
          ],
          "firstName": "string",
          "department": "string",
          "lastName": "string",
          "organizationalUnit": "string",
          "jobTitle": "string",
          "locality": "string",
          "email": "string",
          "stateOrProvince": "string",
          "telephone": "string",
          "country": "string",
          "streetAddress": "string",
          "schema": "string"
        },
        "adAttributes": {
          "attributes": [
            {
              "name": "string",
              "type": "string",
              "internalName": "string",
              "defaultValue": "string"
            }
          ]
        },
        "adScopesNames": "string",
        "link": {
          "rel": "string",
          "href": "string",
          "type": "string"
        }
      }
    ]
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Ansible module to manage CheckPoint Firewall (c) 2019
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
#

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {
    "metadata_version": "1.1",
    "status": ["deprecated"],
    "supported_by": "community",
}

DOCUMENTATION = """
---
module: cp_mgmt_show_tasks
short_description: Retrieve all tasks and show their progress and details.
description:
  - Retrieve all tasks and show their progress and details.
  - All operations are performed over Web Services API.
version_added: "2.0.0"
author: "Or Soffer (@chkp-orso)"
deprecated:
  alternative: cp_mgmt_task_facts
  why: Newer single facts module released.
  removed_at_date: '2024-11-01'
options:
  initiator:
    description:
      - Initiator's name. If name isn't specified, tasks from all initiators will be shown.
    type: str
  status:
    description:
      - Status.
    type: str
    choices: ['successful', 'failed', 'in-progress', 'all']
  from_date:
    description:
      - The date from which tracking tasks is to be performed, by the task's last update date. ISO 8601. If timezone isn't specified in the input, the
        Management server's timezone is used.
    type: str
  to_date:
    description:
      - The date until which tracking tasks is to be performed, by the task's last update date. ISO 8601. If timezone isn't specified in the input,
        the Management server's timezone is used.
    type: str
  limit:
    description:
      - The maximal number of returned results.
    type: int
  offset:
    description:
      - Number of the results to initially skip.
    type: int
  order:
    description:
      - Sorts results by the given field. By default the results are sorted in the descending order by the task's last update date.
    type: list
    elements: dict
    suboptions:
      ASC:
        description:
          - Sorts results by the given field in ascending order.
        type: str
        choices: ['name']
      DESC:
        description:
          - Sorts results by the given field in descending order.
        type: str
        choices: ['name']
  details_level:
    description:
      - The level of detail for some of the fields in the response can vary from showing only the UID value of the object to a fully detailed
        representation of the object.
    type: str
    choices: ['uid', 'standard', 'full']
extends_documentation_fragment: check_point.mgmt.checkpoint_commands
"""

EXAMPLES = """
- name: show-tasks
  cp_mgmt_show_tasks:
    from_date: '2018-05-23T08:00:00'
    initiator: admin1
    status: successful
"""

RETURN = """
cp_mgmt_show_tasks:
  description: The checkpoint show-tasks output.
  returned: always.
  type: dict
"""

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.check_point.mgmt.plugins.module_utils.checkpoint import (
    checkpoint_argument_spec_for_commands,
    api_command,
)


def main():
    argument_spec = dict(
        initiator=dict(type="str"),
        status=dict(
            type="str", choices=["successful", "failed", "in-progress", "all"]
        ),
        from_date=dict(type="str"),
        to_date=dict(type="str"),
        limit=dict(type="int"),
        offset=dict(type="int"),
        order=dict(
            type="list",
            elements="dict",
            options=dict(
                ASC=dict(type="str", choices=["name"]),
                DESC=dict(type="str", choices=["name"]),
            ),
        ),
        details_level=dict(type="str", choices=["uid", "standard", "full"]),
    )
    argument_spec.update(checkpoint_argument_spec_for_commands)

    module = AnsibleModule(argument_spec=argument_spec)

    command = "show-tasks"

    result = api_command(module, command)
    module.exit_json(**result)


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
# Copyright 2019 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

# utils


from __future__ import absolute_import, division, print_function


__metaclass__ = type
from functools import total_ordering

from ansible.module_utils._text import to_text
from ansible.module_utils.basic import missing_required_lib
from ansible.module_utils.common.network import is_masklen, to_netmask
from ansible.module_utils.six import iteritems
from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.utils import (
    dict_diff,
    search_obj_in_list,
)


try:
    import ipaddress

    HAS_IPADDRESS = True
except ImportError:
    HAS_IPADDRESS = False


def remove_command_from_config_list(interface, cmd, commands):
    # To delete the passed config
    if interface not in commands:
        commands.insert(0, interface)
    commands.append("no %s" % cmd)
    return commands


def add_command_to_config_list(interface, cmd, commands):
    # To set the passed config
    if interface not in commands:
        commands.insert(0, interface)
    commands.append(cmd)


def dict_to_set(sample_dict):
    # Generate a set with passed dictionary for comparison
    test_dict = {}
    if isinstance(sample_dict, dict):
        for k, v in iteritems(sample_dict):
            if v is not None:
                if isinstance(v, list):
                    if isinstance(v[0], dict):
                        li = []
                        for each in v:
                            for key, value in iteritems(each):
                                if isinstance(value, list):
                                    each[key] = tuple(value)
                            li.append(tuple(iteritems(each)))
                        v = tuple(li)
                    else:
                        v = tuple(v)
                elif isinstance(v, dict):
                    li = []
                    for key, value in iteritems(v):
                        if isinstance(value, list):
                            v[key] = tuple(value)
                    li.extend(tuple(iteritems(v)))
                    v = tuple(li)
                test_dict.update({k: v})
        return_set = set(tuple(iteritems(test_dict)))
    else:
        return_set = set(sample_dict)
    return return_set


def filter_dict_having_none_value(want, have):
    # Generate dict with have dict value which is None in want dict
    test_dict = dict()
    test_key_dict = dict()
    name = want.get("name")
    if name:
        test_dict["name"] = name
    diff_ip = False
    want_ip = ""
    for k, v in iteritems(want):
        if isinstance(v, dict):
            for key, value in iteritems(v):
                if value is None and k in have and key in have.get(k):
                    dict_val = have.get(k).get(key)
                    test_key_dict.update({key: dict_val})
                test_dict.update({k: test_key_dict})
        if isinstance(v, list) and isinstance(v[0], dict):
            for key, value in iteritems(v[0]):
                if value is None and k in have and key in have.get(k):
                    dict_val = have.get(k).get(key)
                    test_key_dict.update({key: dict_val})
                test_dict.update({k: test_key_dict})
            # below conditions checks are added to check if
            # secondary IP is configured, if yes then delete
            # the already configured IP if want and have IP
            # is different else if it's same no need to delete
            for each in v:
                if each.get("secondary"):
                    want_ip = each.get("address").split("/")
                    have_ip = have.get("ipv4")
                    for each in have_ip:
                        if len(want_ip) > 1 and each.get("secondary"):
                            have_ip = each.get("address").split("/")[0]
                            if have_ip != want_ip[0]:
                                diff_ip = True
                    if each.get("secondary") and diff_ip is True:
                        test_key_dict.update({"secondary": True})
                    test_dict.update({"ipv4": test_key_dict})
                # Checks if want doesn't have secondary IP but have has secondary IP set
                elif have.get("ipv4"):
                    if [True for each_have in have.get("ipv4") if "secondary" in each_have]:
                        test_dict.update({"ipv4": {"secondary": True}})
        if k == "l2protocol":
            diff = True
            h_proto = have.get("l2protocol")
            w_proto = want.get("l2protocol")
            if h_proto:
                if w_proto:
                    for h in h_proto:
                        for w in w_proto:
                            if h == w:
                                diff = False
                        if not diff:
                            break
                else:
                    diff = True
            if diff:
                test_dict.update({k: v})
        if v is None:
            val = have.get(k)
            test_dict.update({k: val})
    return test_dict


def remove_duplicate_interface(commands):
    # Remove duplicate interface from commands
    set_cmd = []
    for each in commands:
        if "interface" in each:
            if each not in set_cmd:
                set_cmd.append(each)
        else:
            set_cmd.append(each)

    return set_cmd


def flatten_dict(x):
    result = {}
    if not isinstance(x, dict):
        return result

    for key, value in iteritems(x):
        if isinstance(value, dict):
            result.update(flatten_dict(value))
        else:
            result[key] = value

    return result


def dict_delete(base, comparable):
    """

    This function generates a dict containing key, value pairs for keys
    that are present in the `base` dict but not present in the `comparable`
    dict.

    :param base: dict object to base the diff on
    :param comparable: dict object to compare against base

    :returns: new dict object with key, value pairs that needs to be deleted.

    """
    to_delete = dict()

    for key in base:
        if isinstance(base[key], dict):
            sub_diff = dict_delete(base[key], comparable.get(key, {}))
            if sub_diff:
                to_delete[key] = sub_diff
        else:
            if key not in comparable:
                to_delete[key] = base[key]

    return to_delete


def pad_commands(commands, interface):
    commands.insert(0, "interface {0}".format(interface))


def diff_list_of_dicts(w, h, key="member"):
    """
    Returns a list containing diff between
    two list of dictionaries
    """
    if not w:
        w = []
    if not h:
        h = []

    diff = []
    for w_item in w:
        h_item = search_obj_in_list(w_item[key], h, key=key) or {}
        d = dict_diff(h_item, w_item)
        if d:
            if key not in d.keys():
                d[key] = w_item[key]
            diff.append(d)

    return diff


def validate_ipv4(value, module):
    if value:
        address = value.split("/")
        if len(address) != 2:
            module.fail_json(
                msg="address format is <ipv4 address>/<mask>, got invalid format {0}".format(
                    value,
                ),
            )

        if not is_masklen(address[1]):
            module.fail_json(
                msg="invalid value for mask: {0}, mask should be in range 0-32".format(
                    address[1],
                ),
            )


def validate_ipv6(value, module):
    if value:
        address = value.split("/")
        if len(address) != 2:
            module.fail_json(
                msg="address format is <ipv6 address>/<mask>, got invalid format {0}".format(
                    value,
                ),
            )
        else:
            if not 0 <= int(address[1]) <= 128:
                module.fail_json(
                    msg="invalid value for mask: {0}, mask should be in range 0-128".format(
                        address[1],
                    ),
                )


def validate_n_expand_ipv4(module, want):
    # Check if input IPV4 is valid IP and expand IPV4 with its subnet mask
    ip_addr_want = want.get("address")
    if len(ip_addr_want.split(" ")) > 1:
        return ip_addr_want
    validate_ipv4(ip_addr_want, module)
    ip = ip_addr_want.split("/")
    if len(ip) == 2:
        ip_addr_want = "{0} {1}".format(ip[0], to_netmask(ip[1]))

    return ip_addr_want


def normalize_interface(name):
    """Return the normalized interface name"""
    if not name:
        return

    def _get_number(name):
        digits = ""
        for char in name:
            if char.isdigit() or char in "/.":
                digits += char
        return digits

    if name.lower().startswith("gi"):
        if_type = "GigabitEthernet"
    elif name.lower().startswith("fa"):
        if_type = "FastEthernet"
    elif name.lower().startswith("for"):
        if_type = "FortyGigE"
    elif name.lower().startswith("te"):
        if_type = "TenGigE"
    elif name.lower().startswith("twe"):
        if_type = "TwentyFiveGigE"
    elif name.lower().startswith("hu"):
        if_type = "HundredGigE"
    elif name.lower().startswith("vl"):
        if_type = "Vlan"
    elif name.lower().startswith("lo"):
        if_type = "Loopback"
    elif name.lower().startswith("bundle-e"):
        if_type = "Bundle-Ether"
    elif name.lower().startswith("bundle-p"):
        if_type = "Bundle-POS"
    elif name.lower().startswith("fif"):
        if_type = "FiftyGigE"
    elif name.lower().startswith("fou"):
        if_type = "FourHundredGigE"
    elif name.lower().startswith("two"):
        if_type = "TwoHundredGigE"
    else:
        if_type = None

    number_list = name.split(" ")
    if len(number_list) == 2:
        number = number_list[-1].strip()
    else:
        number = _get_number(name)
    if if_type:
        proper_interface = if_type + number
    else:
        proper_interface = name

    return proper_interface


def get_interface_type(interface):
    """Gets the type of interface"""

    if interface.upper().startswith("GI"):
        return "GigabitEthernet"
    elif interface.upper().startswith("FA"):
        return "FastEthernet"
    elif interface.upper().startswith("FORT"):
        return "FortyGigE"
    elif interface.upper().startswith("ET"):
        return "Ethernet"
    elif interface.upper().startswith("LO"):
        return "Loopback"
    elif interface.upper().startswith("BUNDLE-E"):
        return "Bundle-Ether"
    elif interface.upper().startswith("BUNDLE-P"):
        return "Bundle-POS"
    elif interface.upper().startswith("NV"):
        return "nve"
    elif interface.upper().startswith("TE"):
        return "TenGigE"
    elif interface.upper().startswith("TWE"):
        return "TwentyFiveGigE"
    elif interface.upper().startswith("HU"):
        return "HundredGigE"
    elif interface.upper().startswith("PRE"):
        return "preconfigure"
    elif interface.upper().startswith("FIF"):
        return "FiftyGigE"
    elif interface.upper().startswith("FOU"):
        return "FourHundredGigE"
    elif interface.upper().startswith("TWO"):
        return "TwoHundredGigE"
    elif interface.upper().startswith("MG"):
        return "management"
    elif interface.upper().startswith("MA"):
        return "management"
    else:
        return "unknown"


def isipaddress(data):
    """
    Checks if the passed string is
    a valid IPv4 or IPv6 address
    """
    if not HAS_IPADDRESS:
        raise Exception(missing_required_lib("ipaddress"))
    isipaddress = True

    try:
        ipaddress.ip_address(data)
    except ValueError:
        isipaddress = False

    return isipaddress


def is_ipv4_address(data):
    """
    Checks if the passed string is
    a valid IPv4 address
    """
    if not HAS_IPADDRESS:
        raise Exception(missing_required_lib("ipaddress"))
    if "/" in data:
        data = data.split("/")[0]

    if not isipaddress(to_text(data)):
        raise ValueError("{0} is not a valid IP address".format(data))

    return ipaddress.ip_address(to_text(data)).version == 4


def prefix_to_address_wildcard(prefix):
    """Converts a IPv4 prefix into address and
        wildcard mask

    :returns: IPv4 address and wildcard mask
    """
    if not HAS_IPADDRESS:
        raise Exception(missing_required_lib("ipaddress"))
    wildcard = []

    subnet = to_text(ipaddress.IPv4Network(to_text(prefix)).netmask)

    for x in subnet.split("."):
        component = 255 - int(x)
        wildcard.append(str(component))

    wildcard = ".".join(wildcard)

    return prefix.split("/")[0], wildcard


def flatten_config(data, context):
    """Flatten different contexts in
        the running-config for easier parsing.
    :param data: dict
    :param context: str
    :returns: flattened running config
    """
    data = data.split("\n")
    in_cxt = False
    cur = {}

    for index, x in enumerate(data):
        cur_indent = len(x) - len(x.lstrip())
        if x.strip().startswith(context):
            in_cxt = True
            cur["context"] = x
            cur["indent"] = cur_indent
        elif cur and (cur_indent <= cur["indent"]):
            in_cxt = False
        elif in_cxt:
            data[index] = cur["context"] + " " + x.strip()
    return "\n".join(data)


@total_ordering
class Version:
    """Simple class to compare arbitrary versions"""

    def __init__(self, version_string):
        self.components = version_string.split(".")

    def __eq__(self, other):
        other = _coerce(other)
        if not isinstance(other, Version):
            return NotImplemented

        return self.components == other.components

    def __lt__(self, other):
        other = _coerce(other)
        if not isinstance(other, Version):
            return NotImplemented

        return self.components < other.components


def _coerce(other):
    if isinstance(other, str):
        other = Version(other)
    if isinstance(other, (int, float)):
        other = Version(str(other))
    return other


def netmask_to_cidr(netmask):
    # convert netmask to cidr and returns the cidr notation
    return str(sum([bin(int(x)).count("1") for x in netmask.split(".")]))



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: node_group_node_create
short_description: Resource module for Node Group Node Create
description:
- Manage operation create of the resource Node Group Node Create.
- This API adds a node to the node group in the cluster. When a node that.
version_added: '2.1.0'
extends_documentation_fragment:
  - cisco.ise.module
author: Rafael Campos (@racampos)
options:
  hostname:
    description: Node Group Node Create's hostname.
    type: str
  nodeGroupName:
    description: NodeGroupName path parameter. Name of the existing node group.
    type: str
requirements:
- ciscoisesdk >= 2.2.3
- python >= 3.5
seealso:
- name: Cisco ISE documentation for Node Group
  description: Complete reference of the Node Group API.
  link: https://developer.cisco.com/docs/identity-services-engine/v1/#!deployment-openapi
notes:
  - SDK Method used are
    node_group.NodeGroup.add_node,

  - Paths used are
    post /api/v1/deployment/node-group/{nodeGroupName}/add-node,

"""

EXAMPLES = r"""
- name: Create
  cisco.ise.node_group_node_create:
    ise_hostname: "{{ise_hostname}}"
    ise_username: "{{ise_username}}"
    ise_password: "{{ise_password}}"
    ise_verify: "{{ise_verify}}"
    hostname: string
    nodeGroupName: string

"""

RETURN = r"""
ise_response:
  description: A dictionary or list with the response returned by the Cisco ISE Python SDK
  returned: always
  type: dict
  sample: >
    {
      "success": {
        "message": "string"
      },
      "version": "string"
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)


DOCUMENTATION = r"""
---
module: backup_plan_info
version_added: 6.0.0
short_description: Describe AWS Backup Plans
description:
  - Lists info about Backup Plan configuration.
author:
  - Gomathi Selvi Srinivasan (@GomathiselviS)
  - Kristof Imre Szabo (@krisek)
  - Alina Buzachis (@alinabuzachis)
options:
  backup_plan_names:
    type: list
    elements: str
    description:
      - Specifies a list of plan names.
extends_documentation_fragment:
  - amazon.aws.common.modules
  - amazon.aws.region.modules
  - amazon.aws.boto3
"""

EXAMPLES = r"""
# Note: These examples do not set authentication details, see the AWS Guide for details.
- name: Gather information about all backup plans
  amazon.aws.backup_plan_info:

- name: Gather information about a particular backup plan
  amazon.aws.backup_plan_info:
    backup plan_names:
      - elastic
"""

RETURN = r"""
backup_plans:
    description: List of backup plan objects. List elements are dict with the information about backup plan.
    type: list
    elements: dict
    returned: always
    contains:
        backup_plan_arn:
            description: An Amazon Resource Name (ARN) that uniquely identifies a backup plan.
            returned: always
            type: str
            sample: arn:aws:backup:eu-central-1:111122223333:backup-plan:1111f877-1ecf-4d79-9718-a861cd09df3b
        backup_plan_id:
            description: Id of the backup plan.
            returned: always
            type: str
            sample: "1111f877-1ecf-4d79-9718-a861cd09df3b"
        backup_plan_name:
            description: The display name of a backup plan.
            returned: always
            type: str
            sample: elastic
        creation_date:
            description: Creation date of the backup plan.
            returned: always
            type: str
            sample: '2023-01-24T10:08:03.193000+01:00'
        last_execution_date:
            description: Last execution date of the backup plan.
            returned: always
            type: str
            sample: '2023-03-24T06:30:08.250000+01:00'
        tags:
            description: Tags of the backup plan
            returned: always
            type: str
            sample: {
                      "TagKey1": "TagValue1",
                      "TagKey2": "TagValue2"
                    }
        version_id:
            description: Version id of the backup plan
            returned: always
            type: str
            sample: 'A2AiAAAmAAAtAxAxAA00AAa0AAAxAxAtA0AmAAA4NDY1ZTZl'
        backup_plan:
            returned: always
            description: Detailed information about the backup plan.
            type: list
            elements: dict
            contains:
                backup_plan_name:
                    description: Name of the backup plan.
                    returned: always
                    type: str
                    sample: elastic
                advanced_backup_settings:
                    description: Advanced backup settings of the backup plan.
                    returned: when configured
                    type: list
                    elements: dict
                    sample: [
                              {
                                "backup_options": {
                                    "windows_vss": "enabled"
                                },
                                "resource_type": "EC2"
                              }
                            ]
                    contains:
                        resource_type:
                            description: Resource type of the advanced setting.
                            type: str
                        backup_options:
                            description: Options of the advanced setting.
                            type: dict
                rules:
                    description:
                      - An array of BackupRule objects, each of which specifies a scheduled task that is used to back up a selection of resources.
                    returned: always
                    type: list
                    elements: dict
                    contains:
                      rule_name:
                          description: A display name for a backup rule.
                          returned: always
                          type: str
                          sample: "daily"
                      target_backup_vault_name:
                          description: The name of a logical container where backups are stored.
                          returned: always
                          type: str
                          sample: "09da67966fd5-backup-vault"
                      schedule_expression:
                          description: A cron expression in UTC specifying when Backup initiates a backup job.
                          returned: always
                          type: str
                          sample: "cron(0 5 ? * * *)"
                      start_window_minutes:
                          description:
                            - A value in minutes after a backup is scheduled before a job will be canceled if it
                              doesn't start successfully.
                          type: int
                          returned: always
                          sample: 480
                      completion_window_minutes:
                          description:
                            - A value in minutes after a backup job is successfully started before it must be
                              completed or it will be canceled by Backup.
                          type: int
                          returned: always
                          sample: 10080
                      lifecycle:
                          description:
                            - Defines when a protected resource is transitioned to cold storage and when it expires.
                          type: dict
                          returned: when configured.
                          sample: {
                                    "delete_after_days": 100,
                                    "move_to_cold_storage_after_days": 10
                                  }
                      recovery_point_tags:
                          description:
                            - An array of key-value pair strings that are assigned to resources that are associated with
                              this rule when restored from backup.
                          type: dict
                          returned: when configured.
                          sample: {
                                    "Tagkey1": "TagValue1",
                                    "Tagkey2": "TagValue2"
                                  }
                      rule_id:
                          description:
                            - Uniquely identifies a rule that is used to schedule the backup of a selection of resources.
                          type: str
                          returned: always
                          sample: "973621ef-d863-41ef-b5c3-9e943a64ad0c"
                      copy_actions:
                          description: An array of CopyAction objects, which contains the details of the copy operation.
                          returned: when configured.
                          type: list
                          elements: dict
                          sample: [
                                    {
                                      "destination_backup_vault_arn": "arn:aws:backup:us-west-2:123456789012:backup-vault:my-test-vault",
                                      "lifecycle": {
                                          "delete_after_days": 100,
                                          "move_to_cold_storage_after_days": 10
                                      }
                                    }
                                  ]
                      enable_continous_backup:
                          description: Specifies whether Backup creates continuous backups.
                          type: bool
                          returned: always
                          sample: false
"""

try:
    import botocore
except ImportError:
    pass  # Handled by AnsibleAWSModule


from ansible_collections.amazon.aws.plugins.module_utils.backup import get_plan_details
from ansible_collections.amazon.aws.plugins.module_utils.modules import AnsibleAWSModule
from ansible_collections.amazon.aws.plugins.module_utils.retries import AWSRetry


def get_all_backup_plans_info(client):
    paginator = client.get_paginator("list_backup_plans")
    return paginator.paginate().build_full_result()


def get_backup_plan_detail(client, module):
    backup_plan_list = []
    backup_plan_names = module.params.get("backup_plan_names")

    if backup_plan_names is None:
        backup_plan_names = []
        backup_plan_list_info = get_all_backup_plans_info(client)["BackupPlansList"]
        for backup_plan in backup_plan_list_info:
            backup_plan_names.append(backup_plan["BackupPlanName"])

    for name in backup_plan_names:
        backup_plan_list.extend(get_plan_details(module, client, name))

    module.exit_json(**{"backup_plans": backup_plan_list})


def main():
    argument_spec = dict(
        backup_plan_names=dict(type="list", elements="str"),
    )

    module = AnsibleAWSModule(argument_spec=argument_spec, supports_check_mode=True)

    try:
        connection = module.client("backup", retry_decorator=AWSRetry.jittered_backoff())
    except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
        module.fail_json_aws(e, msg="Failed to connect to AWS")

    get_backup_plan_detail(connection, module)


if __name__ == "__main__":
    main()



#!/usr/bin/python
#
# Copyright (c) 2018 Yunge Zhu <yungez@microsoft.com>
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

DOCUMENTATION = '''
---
module: azure_rm_virtualnetworkpeering
version_added: "0.0.1"
short_description: Manage Azure Virtual Network Peering
description:
    - Create, update and delete Azure Virtual Network Peering.

options:
    resource_group:
        description:
            - Name of a resource group where the vnet exists.
        required: true
        type: str
    name:
        description:
            - Name of the virtual network peering.
        required: true
        type: str
    virtual_network:
        description:
            - Name or resource ID of the virtual network to be peered.
        type: raw
        required: true
    remote_virtual_network:
        description:
            - Remote virtual network to be peered.
            - It can be name of remote virtual network in same resource group.
            - It can be remote virtual network resource ID.
            - It can be a dict which contains I(name) and I(resource_group) of remote virtual network.
            - Required when creating.
        type: raw
    allow_virtual_network_access:
        description:
            - Allows VMs in the remote VNet to access all VMs in the local VNet.
        type: bool
        default: false
    allow_forwarded_traffic:
        description:
            - Allows forwarded traffic from the VMs in the remote VNet.
        type: bool
        default: false
    use_remote_gateways:
        description:
            - If remote gateways can be used on this virtual network.
        type: bool
        default: false
    allow_gateway_transit:
        description:
            - Allows VNet to use the remote VNet's gateway. Remote VNet gateway must have --allow-gateway-transit enabled for remote peering.
            - Only 1 peering can have this flag enabled. Cannot be set if the VNet already has a gateway.
        type: bool
        default: false
    state:
        description:
            - State of the virtual network peering. Use C(present) to create or update a peering and C(absent) to delete it.
        default: present
        type: str
        choices:
            - absent
            - present

extends_documentation_fragment:
    - azure.azcollection.azure

author:
    - Yunge Zhu (@yungezz)
'''

EXAMPLES = '''
- name: Create virtual network peering
  azure_rm_virtualnetworkpeering:
    resource_group: myResourceGroup
    virtual_network: myVirtualNetwork
    name: myPeering
    remote_virtual_network:
      resource_group: mySecondResourceGroup
      name: myRemoteVirtualNetwork
    allow_virtual_network_access: false
    allow_forwarded_traffic: true

- name: Delete the virtual network peering
  azure_rm_virtualnetworkpeering:
    resource_group: myResourceGroup
    virtual_network: myVirtualNetwork
    name: myPeering
    state: absent
'''
RETURN = '''
id:
    description:
        - ID of the Azure virtual network peering.
    returned: always
    type: str
    sample: "/subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/myResourceGroup/providers/Microsoft.Network/virtualNetworks/myVirtualN
             etwork/virtualNetworkPeerings/myPeering"
peering_sync_level:
    description:
        - The Sync Level of the Peering
    type: str
    returned: always
    sample: "FullyInSync"
'''

try:
    from azure.mgmt.core.tools import is_valid_resource_id
    from azure.core.exceptions import ResourceNotFoundError
    from azure.core.polling import LROPoller
except ImportError:
    # This is handled in azure_rm_common
    pass

from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common import AzureRMModuleBase, format_resource_id


def virtual_network_to_dict(vnet):
    '''
    Convert a virtual network object to a dict.
    '''
    results = dict(
        id=vnet.id,
        name=vnet.name,
        location=vnet.location,
        type=vnet.type,
        tags=vnet.tags,
        provisioning_state=vnet.provisioning_state,
        etag=vnet.etag
    )
    if vnet.dhcp_options and len(vnet.dhcp_options.dns_servers) > 0:
        results['dns_servers'] = []
        for server in vnet.dhcp_options.dns_servers:
            results['dns_servers'].append(server)
    if vnet.address_space and len(vnet.address_space.address_prefixes) > 0:
        results['address_prefixes'] = []
        for space in vnet.address_space.address_prefixes:
            results['address_prefixes'].append(space)
    return results


def vnetpeering_to_dict(vnetpeering):
    '''
    Convert a virtual network peering object to a dict.
    '''
    results = dict(
        id=vnetpeering.id,
        name=vnetpeering.name,
        remote_virtual_network=vnetpeering.remote_virtual_network.id,
        remote_address_space=dict(
            address_prefixes=vnetpeering.remote_address_space.address_prefixes
        ),
        peering_state=vnetpeering.peering_state,
        provisioning_state=vnetpeering.provisioning_state,
        use_remote_gateways=vnetpeering.use_remote_gateways,
        allow_gateway_transit=vnetpeering.allow_gateway_transit,
        allow_forwarded_traffic=vnetpeering.allow_forwarded_traffic,
        allow_virtual_network_access=vnetpeering.allow_virtual_network_access,
        etag=vnetpeering.etag,
        peering_sync_level=vnetpeering.peering_sync_level
    )
    return results


class AzureRMVirtualNetworkPeering(AzureRMModuleBase):

    def __init__(self):
        self.module_arg_spec = dict(
            resource_group=dict(
                type='str',
                required=True
            ),
            name=dict(
                type='str',
                required=True
            ),
            virtual_network=dict(
                type='raw',
                required=True
            ),
            remote_virtual_network=dict(
                type='raw'
            ),
            allow_virtual_network_access=dict(
                type='bool',
                default=False
            ),
            allow_forwarded_traffic=dict(
                type='bool',
                default=False
            ),
            allow_gateway_transit=dict(
                type='bool',
                default=False
            ),
            use_remote_gateways=dict(
                type='bool',
                default=False
            ),
            state=dict(
                type='str',
                default='present',
                choices=['present', 'absent']
            )
        )

        self.resource_group = None
        self.name = None
        self.virtual_network = None
        self.remote_virtual_network = None
        self.allow_virtual_network_access = None
        self.allow_forwarded_traffic = None
        self.allow_gateway_transit = None
        self.use_remote_gateways = None

        self.results = dict(changed=False)

        super(AzureRMVirtualNetworkPeering, self).__init__(derived_arg_spec=self.module_arg_spec,
                                                           supports_check_mode=True,
                                                           supports_tags=False)

    def exec_module(self, **kwargs):
        """Main module execution method"""

        for key in list(self.module_arg_spec.keys()):
            setattr(self, key, kwargs[key])

        to_be_updated = False
        to_be_synced = False

        resource_group = self.get_resource_group(self.resource_group)

        # parse virtual_network
        self.virtual_network = self.parse_resource_to_dict(self.virtual_network)
        if self.virtual_network['resource_group'] != self.resource_group:
            self.fail('Resource group of virtual_network is not same as param resource_group')

        # parse remote virtual_network
        self.remote_virtual_network = self.format_vnet_id(self.remote_virtual_network)

        # get vnet peering
        response = self.get_vnet_peering()

        if self.state == 'present':
            if response:
                # check vnet id not changed
                existing_vnet = self.parse_resource_to_dict(response['id'])
                if existing_vnet['resource_group'] != self.virtual_network['resource_group'] or \
                   existing_vnet['name'] != self.virtual_network['name']:
                    self.fail("Cannot update virtual_network of Virtual Network Peering!")

                # check remote vnet id not changed
                if response['remote_virtual_network'].lower() != self.remote_virtual_network.lower():
                    self.fail("Cannot update remote_virtual_network of Virtual Network Peering!")

                # check if update
                to_be_updated = self.check_update(response)
                to_be_synced = self.check_sync(response)

            else:
                # not exists, create new vnet peering
                to_be_updated = True

                # check if vnet exists
                virtual_network = self.get_vnet(self.virtual_network['resource_group'], self.virtual_network['name'])
                if not virtual_network:
                    self.fail("Virtual network {0} in resource group {1} does not exist!".format(
                        self.virtual_network['name'], self.virtual_network['resource_group']))

        elif self.state == 'absent':
            if response:
                self.log('Delete Azure Virtual Network Peering')
                self.results['changed'] = True
                self.results['id'] = response['id']

                if self.check_mode:
                    return self.results

                response = self.delete_vnet_peering()

            else:
                self.log("Azure Virtual Network Peering {0} does not exist in resource group {1}".format(self.name, self.resource_group))

        if to_be_updated:
            self.results['changed'] = True

            if self.check_mode:
                return self.results

            response = self.create_or_update_vnet_peering()
            self.results['id'] = response['id']
            to_be_synced = self.check_sync(response)

        if to_be_synced:
            self.results['changed'] = True

            if self.check_mode:
                return self.results
            sync_response = self.sync_vnet_peering()
            self.results['peering_sync_level'] = sync_response['peering_sync_level']

        return self.results

    def format_vnet_id(self, vnet):
        if not vnet:
            return vnet
        if isinstance(vnet, dict) and vnet.get('name') and vnet.get('resource_group'):
            remote_vnet_id = format_resource_id(vnet['name'],
                                                self.subscription_id,
                                                'Microsoft.Network',
                                                'virtualNetworks',
                                                vnet['resource_group'])
        elif isinstance(vnet, str):
            if is_valid_resource_id(vnet):
                remote_vnet_id = vnet
            else:
                remote_vnet_id = format_resource_id(vnet,
                                                    self.subscription_id,
                                                    'Microsoft.Network',
                                                    'virtualNetworks',
                                                    self.resource_group)
        else:
            self.fail("remote_virtual_network could be a valid resource id, dict of name and resource_group, name of virtual network in same resource group.")
        return remote_vnet_id

    def check_sync(self, exisiting_vnet_peering):
        if exisiting_vnet_peering['peering_sync_level'] == 'LocalNotInSync':
            return True
        return False

    def check_update(self, exisiting_vnet_peering):
        if self.allow_forwarded_traffic != exisiting_vnet_peering['allow_forwarded_traffic']:
            return True
        if self.allow_gateway_transit != exisiting_vnet_peering['allow_gateway_transit']:
            return True
        if self.allow_virtual_network_access != exisiting_vnet_peering['allow_virtual_network_access']:
            return True
        if self.use_remote_gateways != exisiting_vnet_peering['use_remote_gateways']:
            return True
        return False

    def get_vnet(self, resource_group, vnet_name):
        '''
        Get Azure Virtual Network
        :return: deserialized Azure Virtual Network
        '''
        self.log("Get the Azure Virtual Network {0}".format(vnet_name))
        vnet = self.network_client.virtual_networks.get(resource_group, vnet_name)

        if vnet:
            results = virtual_network_to_dict(vnet)
            return results
        return False

    def sync_vnet_peering(self):
        '''
        Creates or Update Azure Virtual Network Peering.

        :return: deserialized Azure Virtual Network Peering instance state dictionary
        '''
        self.log("Creating or Updating the Azure Virtual Network Peering {0}".format(self.name))

        vnet_id = format_resource_id(self.virtual_network['name'],
                                     self.subscription_id,
                                     'Microsoft.Network',
                                     'virtualNetworks',
                                     self.virtual_network['resource_group'])
        peering = self.network_models.VirtualNetworkPeering(
            id=vnet_id,
            name=self.name,
            remote_virtual_network=self.network_models.SubResource(id=self.remote_virtual_network),
            allow_virtual_network_access=self.allow_virtual_network_access,
            allow_gateway_transit=self.allow_gateway_transit,
            allow_forwarded_traffic=self.allow_forwarded_traffic,
            use_remote_gateways=self.use_remote_gateways
        )

        try:
            response = self.network_client.virtual_network_peerings.begin_create_or_update(self.resource_group,
                                                                                           self.virtual_network['name'],
                                                                                           self.name,
                                                                                           peering,
                                                                                           sync_remote_address_space=True)
            if isinstance(response, LROPoller):
                response = self.get_poller_result(response)
            return vnetpeering_to_dict(response)
        except Exception as exc:
            self.fail("Error creating Azure Virtual Network Peering: {0}.".format(exc.message))

    def create_or_update_vnet_peering(self):
        '''
        Creates or Update Azure Virtual Network Peering.

        :return: deserialized Azure Virtual Network Peering instance state dictionary
        '''
        self.log("Creating or Updating the Azure Virtual Network Peering {0}".format(self.name))

        vnet_id = format_resource_id(self.virtual_network['name'],
                                     self.subscription_id,
                                     'Microsoft.Network',
                                     'virtualNetworks',
                                     self.virtual_network['resource_group'])
        peering = self.network_models.VirtualNetworkPeering(
            id=vnet_id,
            name=self.name,
            remote_virtual_network=self.network_models.SubResource(id=self.remote_virtual_network),
            allow_virtual_network_access=self.allow_virtual_network_access,
            allow_gateway_transit=self.allow_gateway_transit,
            allow_forwarded_traffic=self.allow_forwarded_traffic,
            use_remote_gateways=self.use_remote_gateways
        )

        try:
            response = self.network_client.virtual_network_peerings.begin_create_or_update(self.resource_group,
                                                                                           self.virtual_network['name'],
                                                                                           self.name,
                                                                                           peering)
            if isinstance(response, LROPoller):
                response = self.get_poller_result(response)
            return vnetpeering_to_dict(response)
        except Exception as exc:
            self.fail("Error creating Azure Virtual Network Peering: {0}.".format(exc.message))

    def delete_vnet_peering(self):
        '''
        Deletes the specified Azure Virtual Network Peering

        :return: True
        '''
        self.log("Deleting Azure Virtual Network Peering {0}".format(self.name))
        try:
            poller = self.network_client.virtual_network_peerings.begin_delete(
                self.resource_group, self.virtual_network['name'], self.name)
            self.get_poller_result(poller)
            return True
        except Exception as e:
            self.fail("Error deleting the Azure Virtual Network Peering: {0}".format(e.message))
            return False

    def get_vnet_peering(self):
        '''
        Gets the Virtual Network Peering.

        :return: deserialized Virtual Network Peering
        '''
        self.log(
            "Checking if Virtual Network Peering {0} is present".format(self.name))
        try:
            response = self.network_client.virtual_network_peerings.get(self.resource_group,
                                                                        self.virtual_network['name'],
                                                                        self.name)
            self.log("Response : {0}".format(response))

            return vnetpeering_to_dict(response)
        except ResourceNotFoundError:
            self.log('Did not find the Virtual Network Peering.')
            return False


def main():
    """Main execution"""
    AzureRMVirtualNetworkPeering()


if __name__ == '__main__':
    main()



#
# -*- coding: utf-8 -*-
# Copyright 2023 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
#

"""
The parse_cli_textfsm filter plugin
"""
from __future__ import absolute_import, division, print_function


__metaclass__ = type

DOCUMENTATION = """
name: parse_cli_textfsm
author: Peter Sprygada (@privateip)
version_added: "1.0.0"
short_description: parse_cli_textfsm filter plugin.
description:
  - The network filters also support parsing the output of a CLI command using the TextFSM library.
    To parse the CLI output with TextFSM use this filter.
  - Using the parameters below - C(data | ansible.netcommon.parse_cli_textfsm(template.yml))
notes:
  - Use of the TextFSM filter requires the TextFSM library to be installed.
options:
  value:
    description:
    - This source data on which parse_cli_textfsm invokes.
    type: raw
    required: True
  template:
    description:
    - The template to compare it with.
    - For example C(data | ansible.netcommon.parse_cli_textfsm(template.yml)),
      in this case C(data) represents this option.
    type: str
"""

EXAMPLES = r"""
# Using parse_cli_textfsm

- name: "Fetch command output"
  cisco.ios.ios_command:
    commands:
      - show lldp neighbors
  register: lldp_output

- name: "Invoke parse_cli_textfsm"
  ansible.builtin.set_fact:
    device_neighbors: "{{ lldp_output.stdout[0] | parse_cli_textfsm('~/ntc-templates/templates/cisco_ios_show_lldp_neighbors.textfsm') }}"

- name: "Debug"
  ansible.builtin.debug:
    msg: "{{ device_neighbors }}"

# Task Output
# -----------
#
# TASK [Fetch command output]
# ok: [rtr-1]

# TASK [Invoke parse_cli_textfsm]
# ok: [rtr-1]

# TASK [Debug]
# ok: [rtr-1] => {
#     "msg": [
#         {
#             "CAPABILITIES": "R",
#             "LOCAL_INTERFACE": "Gi0/0",
#             "NEIGHBOR": "rtr-3",
#             "NEIGHBOR_INTERFACE": "Gi0/0"
#         },
#         {
#             "CAPABILITIES": "R",
#             "LOCAL_INTERFACE": "Gi0/1",
#             "NEIGHBOR": "rtr-1",
#             "NEIGHBOR_INTERFACE": "Gi0/1"
#         }
#     ]
# }
"""

from ansible.errors import AnsibleFilterError
from ansible_collections.ansible.utils.plugins.module_utils.common.argspec_validate import (
    AnsibleArgSpecValidator,
)

from ansible_collections.ansible.netcommon.plugins.plugin_utils.parse_cli_textfsm import (
    parse_cli_textfsm,
)


try:
    from jinja2.filters import pass_environment
except ImportError:
    from jinja2.filters import environmentfilter as pass_environment


@pass_environment
def _parse_cli_textfsm(*args, **kwargs):
    """Extend vlan data"""

    keys = ["value", "template"]
    data = dict(zip(keys, args[1:]))
    data.update(kwargs)
    aav = AnsibleArgSpecValidator(data=data, schema=DOCUMENTATION, name="parse_cli_textfsm")
    valid, errors, updated_data = aav.validate()
    if not valid:
        raise AnsibleFilterError(errors)
    return parse_cli_textfsm(**updated_data)


class FilterModule(object):
    """parse_cli_textfsm"""

    def filters(self):
        """a mapping of filter names to functions"""
        return {"parse_cli_textfsm": _parse_cli_textfsm}



#!/usr/bin/python
# coding: utf-8 -*-

# (c) 2017, Wayne Witzel III <wayne@riotousliving.com>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type


ANSIBLE_METADATA = {'metadata_version': '1.1', 'status': ['preview'], 'supported_by': 'community'}


DOCUMENTATION = '''
---
module: organization
author: "Wayne Witzel III (@wwitzel3)"
short_description: create, update, or destroy Automation Platform Controller organizations
description:
    - Create, update, or destroy Automation Platform Controller organizations. See
      U(https://www.ansible.com/tower) for an overview.
options:
    name:
      description:
        - Name to use for the organization.
      required: True
      type: str
    new_name:
      description:
        - Setting this option will change the existing name (looked up via the name field.
      type: str
    description:
      description:
        - The description to use for the organization.
      type: str
    default_environment:
      description:
        - Default Execution Environment name, ID, or named URL to use for jobs owned by the Organization.
      type: str
    custom_virtualenv:
      description:
        - Local absolute file path containing a custom Python virtualenv to use.
        - Only compatible with older versions of AWX/Tower
        - Deprecated, will be removed in the future
      type: str
    max_hosts:
      description:
        - The max hosts allowed in this organizations
      type: int
    state:
      description:
        - Desired state of the resource.
      default: "present"
      choices: ["present", "absent", "exists"]
      type: str
    instance_groups:
      description:
        - list of Instance Group names, IDs, or named URLs for this Organization to run on.
      type: list
      elements: str
    notification_templates_started:
      description:
        - list of notifications to send on start
      type: list
      elements: str
    notification_templates_success:
      description:
        - list of notifications to send on success
      type: list
      elements: str
    notification_templates_error:
      description:
        - list of notifications to send on error
      type: list
      elements: str
    notification_templates_approvals:
      description:
        - list of notifications to send on start
      type: list
      elements: str
    galaxy_credentials:
      description:
        - list of Ansible Galaxy credential names, IDs, or named URLs to associate to the organization
      type: list
      elements: str
extends_documentation_fragment: awx.awx.auth
'''


EXAMPLES = '''
- name: Create organization
  organization:
    name: "Foo"
    description: "Foo bar organization"
    state: present
    controller_config_file: "~/tower_cli.cfg"

- name: Create organization using 'foo-venv' as default Python virtualenv
  organization:
    name: "Foo"
    description: "Foo bar organization using foo-venv"
    state: present
    controller_config_file: "~/tower_cli.cfg"

- name: Create organization that pulls content from galaxy.ansible.com
  organization:
    name: "Foo"
    state: present
    galaxy_credentials:
      - Ansible Galaxy
    controller_config_file: "~/tower_cli.cfg"
'''

from ..module_utils.controller_api import ControllerAPIModule


def main():
    # Any additional arguments that are not fields of the item can be added here
    argument_spec = dict(
        name=dict(required=True),
        new_name=dict(),
        description=dict(),
        default_environment=dict(),
        custom_virtualenv=dict(),
        max_hosts=dict(type='int'),
        instance_groups=dict(type="list", elements='str'),
        notification_templates_started=dict(type="list", elements='str'),
        notification_templates_success=dict(type="list", elements='str'),
        notification_templates_error=dict(type="list", elements='str'),
        notification_templates_approvals=dict(type="list", elements='str'),
        galaxy_credentials=dict(type="list", elements='str'),
        state=dict(choices=['present', 'absent', 'exists'], default='present'),
    )

    # Create a module for ourselves
    module = ControllerAPIModule(argument_spec=argument_spec)

    # Extract our parameters
    name = module.params.get('name')
    new_name = module.params.get("new_name")
    description = module.params.get('description')
    default_ee = module.params.get('default_environment')
    custom_virtualenv = module.params.get('custom_virtualenv')
    max_hosts = module.params.get('max_hosts')
    state = module.params.get('state')

    # Attempt to look up organization based on the provided name
    organization = module.get_one('organizations', name_or_id=name, check_exists=(state == 'exists'))

    if state == 'absent':
        # If the state was absent we can let the module delete it if needed, the module will handle exiting from this
        module.delete_if_needed(organization)
    # Attempt to look up associated field items the user specified.
    association_fields = {}

    instance_group_names = module.params.get('instance_groups')
    if instance_group_names is not None:
        association_fields['instance_groups'] = []
        for item in instance_group_names:
            association_fields['instance_groups'].append(module.resolve_name_to_id('instance_groups', item))

    notifications_start = module.params.get('notification_templates_started')
    if notifications_start is not None:
        association_fields['notification_templates_started'] = []
        for item in notifications_start:
            association_fields['notification_templates_started'].append(module.resolve_name_to_id('notification_templates', item))

    notifications_success = module.params.get('notification_templates_success')
    if notifications_success is not None:
        association_fields['notification_templates_success'] = []
        for item in notifications_success:
            association_fields['notification_templates_success'].append(module.resolve_name_to_id('notification_templates', item))

    notifications_error = module.params.get('notification_templates_error')
    if notifications_error is not None:
        association_fields['notification_templates_error'] = []
        for item in notifications_error:
            association_fields['notification_templates_error'].append(module.resolve_name_to_id('notification_templates', item))

    notifications_approval = module.params.get('notification_templates_approvals')
    if notifications_approval is not None:
        association_fields['notification_templates_approvals'] = []
        for item in notifications_approval:
            association_fields['notification_templates_approvals'].append(module.resolve_name_to_id('notification_templates', item))

    galaxy_credentials = module.params.get('galaxy_credentials')
    if galaxy_credentials is not None:
        association_fields['galaxy_credentials'] = []
        for item in galaxy_credentials:
            association_fields['galaxy_credentials'].append(module.resolve_name_to_id('credentials', item))

    # Create the data that gets sent for create and update
    org_fields = {
        'name': new_name if new_name else (module.get_item_name(organization) if organization else name),
    }
    if description is not None:
        org_fields['description'] = description
    if default_ee is not None:
        org_fields['default_environment'] = module.resolve_name_to_id('execution_environments', default_ee)
    if custom_virtualenv is not None:
        org_fields['custom_virtualenv'] = custom_virtualenv
    if max_hosts is not None:
        org_fields['max_hosts'] = max_hosts

    # If the state was present and we can let the module build or update the existing organization, this will return on its own
    module.create_or_update_if_needed(
        organization,
        org_fields,
        endpoint='organizations',
        item_type='organization',
        associations=association_fields,
    )


if __name__ == '__main__':
    main()



#!/usr/bin/python
#
# Copyright (c) 2021 Ross Bender (@l3ender)
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
---
module: azure_rm_webappvnetconnection_info

version_added: "1.8.0"

short_description: Get Azure web app virtual network connection facts

description:
    - Get facts for a web app's virtual network connection.

options:
    name:
        description:
            - Name of the web app.
        required: true
        type: str
    resource_group:
        description:
            - Resource group of the web app.
        required: true
        type: str

extends_documentation_fragment:
    - azure.azcollection.azure

author:
    - Ross Bender (@l3ender)
'''

EXAMPLES = '''
- name: Get web app virtual network connection
  azure_rm_webappvnetconnection_info:
    name: "MyWebapp"
    resource_group: "MyResourceGroup"
'''

RETURN = '''
connection:
    description:
        - The web app's virtual network connection.
    returned: always
    type: complex
    contains:
        id:
            description:
                - ID of the web app virtual network connection.
            returned: always
            type: str
            sample: /subscriptions/xxx-xxx/resourceGroups/myResourceGroup/providers/Microsoft.Web/sites/myWebApp/virtualNetworkConnections/yyy-yyy_subnet
        name:
            description:
                - Name of the web app virtual network connection.
            returned: always
            type: str
            sample: yyy-yyy_subnet
        subnet_name:
            description:
               - Name of the subnet connected to the web app.
            returned: always
            type: str
            sample: mySubnet
        vnet_name:
            description:
               - Name of the virtual network connected to the web app.
            returned: always
            type: str
            sample: myVnet
        vnet_resource_group:
            description:
               - Name of the resource group the virtual network is in.
            returned: always
            type: str
            sample: myResourceGroup
        vnet_resource_id:
            description:
               - ID of the virtual network/subnet connected to the web app.
            returned: always
            type: str
            sample: /subscriptions/xxx-xxx/resourceGroups/myResourceGroup/providers/Microsoft.Network/virtualNetworks/myVnet/subnets/mySubnet
'''

from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common import AzureRMModuleBase


class AzureRMWebAppVnetConnectionInfo(AzureRMModuleBase):

    def __init__(self):

        self.module_arg_spec = dict(
            name=dict(type='str', required=True),
            resource_group=dict(type='str', required=True),
        )

        self.results = dict(
            changed=False,
            connection=dict(),
        )

        self.name = None
        self.resource_group = None

        super(AzureRMWebAppVnetConnectionInfo, self).__init__(self.module_arg_spec,
                                                              supports_check_mode=True,
                                                              supports_tags=False,
                                                              facts_module=True)

    def exec_module(self, **kwargs):
        for key in self.module_arg_spec:
            setattr(self, key, kwargs[key])

        vnet = self.get_vnet_connection()

        if vnet:
            self.results['connection'] = self.set_results(vnet)

        return self.results

    def get_vnet_connection(self):
        connections = self.list_vnet_connections()
        for connection in connections:
            if connection.is_swift:
                return connection

        return None

    def list_vnet_connections(self):
        try:
            return self.web_client.web_apps.list_vnet_connections(resource_group_name=self.resource_group, name=self.name)
        except Exception as exc:
            self.fail("Error getting webapp vnet connections {0} (rg={1}) - {2}".format(self.name, self.resource_group, str(exc)))

    def set_results(self, vnet):
        vnet_dict = vnet.as_dict()

        output = dict()
        output['id'] = vnet_dict['id']
        output['name'] = vnet_dict['name']
        subnet_id = vnet_dict['vnet_resource_id']
        output['vnet_resource_id'] = subnet_id
        subnet_detail = self.get_subnet_detail(subnet_id)
        output['vnet_resource_group'] = subnet_detail['resource_group']
        output['vnet_name'] = subnet_detail['vnet_name']
        output['subnet_name'] = subnet_detail['subnet_name']

        return output


def main():
    AzureRMWebAppVnetConnectionInfo()


if __name__ == '__main__':
    main()
